unit ICGreyBoxWhats;

(*	This file is part of the Internet Configuration system and is placed in the public domain for the benefit of all.

	This module implements the Grey Box What, an extraordinarily simple What
	that just draws a grey box.  This is used as an item delimiter in various
	IC dialogs.
*)

interface

	uses
		ICWindowGlobals;

	(* ***** Standard What Interface ***** *)
	
	(* See the comments in ICWindowGlobals. *)

	function WhatInitGreyBox(dummy : integer) : OSStatus;
	function WhatOpenGreyBox (wt: WindowType; item: integer): OSStatus;

	var
		gGreyBoxUserItemUpdateUPP : UserItemUPP;

		(* Exported for the benefit of others who need grey box user items. *)

implementation

	uses
		Appearance,
		
		ICDebug,
		ICDialogs,
		ICGlobals,
		ICMiscSubs;

	(* ***** Drawing ***** *)

	procedure GreyBoxUserItemUpdateProc (window: DialogPtr; item: integer);
		(* This routine is called by the Dialog Manager to redraw the grey
			box user item.
		*)
		var
			itemRect : Rect;
			junk : OSStatus;
			high : Boolean;
			wide : Boolean;
	begin
		GetDItemRect(window, item, itemRect);
		if HasAppearance then begin
			high := (itemRect.bottom - itemRect.top) > 2;
			wide := (itemRect.right - itemRect.left) > 2;
			if high = wide then begin
				junk := DrawThemePrimaryGroup(itemRect, kThemeStateActive);
			end else begin
				junk := DrawThemeSeparator(itemRect, kThemeStateActive);
			end; (* if *)
			ICAssert(junk = noErr);
		end else begin
			DrawGreyBox(itemRect);
		end; (* if *)
	end; (* GreyBoxUserItemUpdateProc *)

	(* ***** Standard What Interface ***** *)

	(* ***** What Init ***** *)

	function WhatInitGreyBox : OSStatus;
	begin
		{$unused dummy}
		gGreyBoxUserItemUpdateUPP := NewUserItemUPP(@GreyBoxUserItemUpdateProc);
		WhatInitGreyBox := noErr;
	end; (* WhatInitGreyBox *)

	(* ***** What Open ***** *)

	function WhatOpenGreyBox (wt: WindowType; item: integer): OSStatus;
		(* See the comments in ICWindowGlobals. *)
	begin
		SetDItemHandle(gWindows[wt].window, item, gGreyBoxUserItemUpdateUPP);
		WhatOpenGreyBox := noErr;
	end; (* WhatOpenGreyBox *)
	
end. (* ICGreyBoxWhats *)