unit ICWhatSubs;

(*	This file is part of the Internet Configuration system and is placed in the public domain for the benefit of all.

	This module contains routines that are common to both of the List Manager based
	Whats, namely File Map What and the Helpers What.  These two whats have so much
	block copied code that it proved relatively easy to move out chunks of them into
	this module.
	
	The API to this module isn't particularly well designed.  It's basically a bunch
	of common routines collected together.
*)

interface

	uses
		Types,
		QuickDraw,
		Lists,

		ICWindowGlobals;

	const
		kListCellHeight = 36;
				
	function CreateCustomListQ(var rView : Rect; var dataBounds : ListBounds; 
								cellSize : Point;
								listDefProc : ListDefUPP; 
								theWindow : WindowPtr; 
								drawIt : Boolean; 
								hasGrow : Boolean; 
								scrollHoriz : Boolean; 
								scrollVert : Boolean; 
								VAR outList : ListHandle) : OSStatus;
		(* 	CreateCustomListQ is a wrapper for Carbon's CreateCustomList
			which works on pre-Carbon systems via the magic of rSimpleLDEF.
			
			It's in this file (instead of ICMiscSubs.p because it needs
			to know about rSimpleLDEF, which is an IC application-specific
			thing.
		*)

	procedure DrawListIcon(iconType : OSType; iconCreator : OSType;
											drawSelected : Boolean; theCellRect : Rect;
											var currentXOffset : integer; iconRegion : RgnHandle);
		(* Draws the file icon associated with a list cell given it's type
			and creator. currentXOffset is adjusted to reflect the size
			of the drawn icon. iconRegion is updated to contain the
			region of the icon drawn.

			This routine assumes that the System 7 icon utilities are
			present and that iconRegion is not nil.
		*)

	procedure HighlightListCell(theCellRect : Rect; iconRegion : RgnHandle);
		(* Draws the highlighting for a list cell by first calculating the
			highlight region based on the cell's rectangle, and then subtracting
			away the iconRegion.
		*)

	procedure ListUserItemUpdateProc (dlg: DialogPtr; item: integer);
		(* This routine is called by the Dialog Manager to update
			the main list user item in the File Mappings dialog. It is installed
			as the user item update proc by the What's Open routine.
			It used to call straight through to the List Manager to do the
			job, but now life is more complex.  When does it ever get simpler?
		*)

	function WhatCursorListItem (wt: WindowType; item: integer; pt: Point; cursorid: integer): OSStatus;
		(* The list Whats (File Map and Helpers) overrides the default cursor handling because
			we want to make sure that the cursor is the big plus within the body of the list,
			but the normal arrow within the scroll bar. Both of these overrides call straight
			through to here because they perform an identical function.
		*)

implementation

	uses
		Appearance,
		Icons,
		{$IFC NOT TARGET_API_MAC_CARBON} ICAccessors, {$ENDC}

		ICDebug,
		ICResources,
		ICMiscSubs,
		ICDialogs,
		ICIconSuites,
		ICThemeState;

	function CreateCustomListQ(var rView : Rect; var dataBounds : ListBounds; 
								cellSize : Point;
								listDefProc : ListDefUPP; 
								theWindow : WindowPtr; 
								drawIt : Boolean; 
								hasGrow : Boolean; 
								scrollHoriz : Boolean; 
								scrollVert : Boolean; 
								VAR outList : ListHandle) : OSStatus;
		(* See comment in interface part. *)
		var
			err : OSStatus;
			{$ifc TARGET_API_MAC_CARBON}
				listDef : ListDefSpec;
			{$endc}
	begin
		{$ifc TARGET_API_MAC_CARBON}
			listDef.defType := kListDefUserProcType;
			listDef.userProc := listDefProc;
			err := CreateCustomList(rView, dataBounds, cellSize, listDef, theWindow, drawIt, hasGrow, scrollHoriz, scrollVert, outList);
			if err <> noErr then begin
				ICAssert(outList = nil);
				outList := nil;
			end; (* if *)
		{$elsec}
			err := noErr;
			outList := LNew(rView, dataBounds, cellSize, rSimpleLDEF, theWindow, drawIt, hasGrow, scrollHoriz, scrollVert);
			if outList = nil then begin
				err := memFullErr;
			end else begin
				SetListRefCon(outList, SInt32(listDefProc));
			end; (* if *)
		{$endc}
		CreateCustomListQ := err;
	end; (* CreateCustomListQ *)
	
	procedure DrawListIcon(iconType : OSType; iconCreator : OSType;
							drawSelected : Boolean; theCellRect : Rect;
							var currentXOffset : integer; iconRegion : RgnHandle);
		(* See comment in interface part. *)
		var
			iconRect: Rect;
			transfer: integer;
	begin
		(* Calculate the destination rectangle for the icon plot. *)
		iconRect := theCellRect;
		iconRect.bottom := iconRect.top + 32;
		iconRect.right := iconRect.left + 32;
		OffsetRect(iconRect, (kListCellHeight - 32) div 2, (kListCellHeight - 32) div 2);

		(* Get the icon data, and plot it. *)

		if drawSelected then begin
			transfer := ttSelected;
		end else begin
			transfer := ttNone;
		end; (* if *)
		PlotIconCached(iconType, iconCreator, iconRect, atNone, transfer, iconRegion);
		
		currentXOffset := currentXOffset + kListCellHeight;
	end; (* DrawListIcon *)
	
	procedure HighlightListCell(theCellRect : Rect; iconRegion : RgnHandle);
		(* See comment in interface part. *)
		var
			highLightRegion: RgnHandle;
	begin
		highLightRegion := NewRgn;
		if highLightRegion <> nil then begin
			RectRgn(highLightRegion, theCellRect);
			MagicMarkerMode;
			if iconRegion <> nil then begin
				XorRgn(iconRegion, highLightRegion, highLightRegion);
			end; (* if *)
			InvertRgn(highLightRegion);
			DisposeRgn(highLightRegion);
		end; (* if *)
	end; (* HighlightListCell *)

	procedure ListUserItemUpdateProc(dlg: DialogPtr; item: integer);
		(* See comment in interface part. *)
		var
			err : OSStatus;
			junk : OSStatus;
			listItemRect: Rect;
			state : MoreThemeDrawingState;
			themeState : ThemeDrawState;
			listH : ListHandle;
			listVisibleCells : ListBounds;
			listDataBounds : ListBounds;
			listCellSize : Point;
			toErase : Rect;
			junkListBoundsPtr : ListBoundsPtr;
			junkRectPtr : RectPtr;
			junkPointPtr : PointPtr;
	begin
		err := MoreGetThemeDrawingState(state);
		if err = noErr then begin
			GetDItemRect(dlg, item, listItemRect);

			// OK, this is a scary chunk of code.  The basic idea is to draw
			// the frame around the list box.  However, the way we do this
			// varies radically depending on whether we have Appearance.
			// If we do, we use Appearance primites to draw the list box
			// frame and the focus rect.  This ensures a) that the list
			// box frame goes the correct colour of grey when the window
			// is inactive, and b) the focus rect meets Appearance standards.
			// Then we normalise the pen.  If we normalise the pen before,
			// the DrawThemeFocusRect(r, false) will erase to white, not
			// the window background grey.
			//
			// On the other hand, if were pre-Appearance, we normalise
			// first, which ensures that the frame is drawn in a single
			// pixel black line.
			
			if HasAppearance then begin
				if GetDialogWindow(dlg) = FrontWindow then begin
					themeState := kThemeStateActive;
				end else begin
					themeState := kThemeStateInactive;
				end; (* if *)
				junk := DrawThemeListBoxFrame(listItemRect, themeState);
				ICAssert(junk = noErr);
				junk := DrawThemeFocusRect(listItemRect, GetDialogWindow(dlg) = FrontWindow);
				ICAssert(junk = noErr);
				junk := MoreNormalizeThemeDrawingState;
				ICAssert(junk = noErr);
			end else begin
				junk := MoreNormalizeThemeDrawingState;
				ICAssert(junk = noErr);
				InsetRect(listItemRect, -1, -1);
				FrameRect(listItemRect);
			end; (* if *)

			listH := ListHandle(gWindows[MapDialogPtrToWindowType(dlg)].items[item]^.whatSpareData);			
			LUpdateWhite(MyGetWindowVisibleRegion(GetDialogWindow(dlg)), listH);

			// Make up for a well-documented lameness in the List Manager.
			// If the list data is smaller than the list view rectangle,
			// the List Manager does not erase any space between the bottom
			// of the last cell of data and the end of the view rectangle.
			// It assumes that the Window Manager has erased this area for
			// it.  But with Appearance, the Window Manager erases this area
			// to grey, not white.  So we have to erase it ourselves.
			// I talked with some HLTB folks at Apple and this is the only
			// real solution to this problem.
			
			junkListBoundsPtr := GetListVisibleCells(listH, listVisibleCells);
			junkListBoundsPtr := GetListDataBounds(listH, listDataBounds);
			if (listVisibleCells.top = 0) & (listDataBounds.bottom < listVisibleCells.bottom) then begin
				junkPointPtr := GetListCellSize(listH, listCellSize);
				junkRectPtr := GetListViewBounds(listH, toErase);
				toErase.top := toErase.top + listDataBounds.bottom * listCellSize.v;
				EraseRect(toErase);
			end; (* if *)
			
			err := MoreSetThemeDrawingState(state, true);
		end; (* if *)
		ICAssert(err = noErr);	
	end; (* ListUserItemUpdateProc *)
		
	function WhatCursorListItem (wt: WindowType; item: integer; pt: Point; cursorid: integer): OSStatus;
		(* See comment in interface part. *)
		var
			listItemRect: Rect;
	begin
		GetDItemRect(gWindows[wt].window, item, listItemRect);
		listItemRect.right := listItemRect.right - 15;
		if PtInRect(pt, listItemRect) then begin
			SetCursor(GetCursor(cursorid)^^);
		end else begin
			InitCursor;
		end; (* if *)
		WhatCursorListItem := noErr;
	end; (* WhatCursorListItem *)

end. (* ICWhatSubs *)