unit ICExtensionResources;

(* Start Header *)

(* File:         ComponentResources.p
 * Generated by: 1.0d8
 * For:          IC 2.1
 * On:           Monday, 4 October 1999, 0:53:14
 * 
 * This file is part of the Internet Configuration system and
 * is placed in the public domain for the benefit of all.
 *)

(* End Header *)

(* This unit holds the selectors for the IC component.  I've moved this into a unit so
	that the override components can access this information without having to
	duplicate it.
*)

interface

	uses
		Types,
		
		InternetConfig,
		
		ICExtensionGlobals;
		
	function SelectorToStr (what: integer): Str32;
	(* This routine converts a IC component into its corresponding identifier
		string.  It is great for debugging because you don't have to remember
		what all of those random selectors are.
	*)

	(* ***** Resources and Resource Management ***** *)

	(*	File Format Notes
		----------------
		The Internet Preferences file format is very simple.  IC is basically
		a thin layer over the Resource Manager.  All IC preferences are
		stored as resources of a specific type.  It turns out that the resource
		type is a direct mapping with the ICProfileID exposed by the API.
		The key for a preference is stored as the resource name.  The attributes
		for a preference are stored as the first 4 bytes of the resource.
		The resource ID is not significant.
		
		In IC 1.4 or later, the file also profits a version number, which
		is the version number of the component that created the default
		preferences in the file.  This version number is not set up by
		the link-in glue.  If a future component successfully updates the
		default preferences, this version will be set to the component's
		version.
		
		In IC 2.0, the file profits a 'CURP' resource, which holds the
		ICProfileID of the current profile.  Also, the preference file
		version is now stored on a profile-by-profile basis.  [Updating
		the preference file is slow.  To avoid taking the hit for updating
		all profiles at once, we only update a profile when you actually
		switch to it.]  There is also a fixed preference (kICProfileName) 
		which contains the profile name.
	*)
	
	(*	
		Resource Type Choice
		--------------------

		Internet Config preferences are stored as resources of
		type 'ICxy', where x is a character in the range
		'R'..'Z' and y is a character in the range 'P'..'Z'.
		Each combination represents a different "profile".
	
		Well, that's how it would work in a perfect world.
		However, the Resource Manager is not a database,
		and has a limit of about 2700 resources.  If we
		allow 99 profiles with, say, 80 keys each, we exceed
		that limit.  So instead I'm artificially limiting
		the current build to only 33 (33 * 80 = 2640) profiles.
	*)
	
	const
		kICPrefResourceTypeBase = 'ICRP';
		kNumberOfDigitsInThirdChar = 3;		(* 'R'..'T' *)
		kNumberOfDigitsInFourthChar = 11;	(* 'P'..'Z' *)
		
	const
		(* STR# *)
		rMiscStrings = 1024;
			strDamaged = 1;
			strDamagedBeyondRepair = 2;
			strBackupInTrash = 3;
			strDamagedSuffix = 4;
			strDefaultFileName = 5;
			strNewConfigPrompt = 6;
			strCopy = 7;				// " copy"
			strCopyI = 8;				// " copy "
			strDefaultSetName = 9;		// "untitled"
			
			(*
				If you alter strDefaultSetName you should also
				alter the 'Pref' resource with key kICProfileName
				because that contains the default name used for
				profiles that are converted from old IC pre-2.0
				preference files.
			
				Aren't comments grand!  I was trying to figure
				out where why I was still getting the old name,
				and then I found this comment.
				-- Quinn, 17 Jul 1998, IC 2.0.1b1
			*)

		(* ALRT *)
		rDamagedAlert = 1024;
		
		(* PROC *)
		rGestaltProc = 129;
			kGestaltProcDataOffset = 2;		// storage for value is 2 bytes into the resource
			
		(* DATA *)
		rICApplicationCreator = 128;
			(*
				If you change this, you should also change
				the similar declaration in the IC application.
				It was too difficult to setup a shared resource
				constant file at this time.
				-- Quinn, 17 Jul 1998, IC 2.0.1b1
			*)
			
		
		(* CURP *)
		rCurrentProfileResource = 128;

		kICProfileName = '49434170ProfileName';
		kICProfileVersion = '49434170ProfileVersion';
	
{$ALIGN MAC68K}

	type
		ICProfileVersionResource =
			record
				attr : ICAttr;
				version : NumVersion;
			end;
		ICProfileVersionResourcePtr = ^ICProfileVersionResource;
		ICProfileVersionResourceHandle = ^ICProfileVersionResourcePtr;

{$ALIGN RESET}
		
	(*	The core implementation regularly needs access to its own resources.
		In some cases, subroutines need access to resources but they don't
		know whether the calling routine has already opened the resource
		file.  [The underlying resource code (Component Manager's 
		OpenComponentResFile routine or FSpOpenResFile make no attempt to 
		reference count the resource file.]  So we build our own reference 
		counting mechanims.
		
		You can use the following two routines in two different ways:
		
			err := StartUsingICResources(didOpen);
			if err = noErr begin
				// do stuff
				StopUsingICResources(didOpen)
			end
		
		or
		
			err := StartUsingICResources(didOpen);
			if err = noErr begin
				// do stuff
			end
			StopUsingICResources(didOpen)
		
		Both of these should work just hunky dory.
	*)

	function StartUsingICResources(globals: ICInstanceGlobalsPtr; var didOpen : Boolean) : ICError;
		(* Makes the component's resources available.  Sets didOpen to true if it actually
			opened the resource file.  If this returns noErr, you should make sure you make
			a matching call to StopUsingICResources.
		*)

	function StopUsingICResources(globals: ICInstanceGlobalsPtr; didOpen : Boolean) : ICError;
		(* Closes the component's resource file if we opened it. *)

	function ICGetIndStringResource (globals: ICInstanceGlobalsPtr; 
									strListID: integer; index: integer; str: StringPtr): ICError;
		(* Get a string from a STR# resource out of the component resource fork.
			One important feature of this routine is that it doesn't modify str unless
			it succeeds in getting a non-empty string.  This is to assist clients
			that go:
				str := 'hardwire constant';
				err := ICGetIndStringResource(...);
			and junk the error code, assuming that the hardwired constant is
			good enough in the presence of errors.
		*)
	
implementation

	uses
		Components,
		Resources,
		TextUtils,
		
		InternetConfig;

	function SelectorToStr (what: integer): Str32;
		(* See comment in interface part. *)
	begin
		case what of
			(* Component Manager stuff *)
			kComponentVersionSelect: 
				SelectorToStr := 'kComponentVersionSelect';
			kComponentCanDoSelect: 
				SelectorToStr := 'kComponentCanDoSelect';
			kComponentOpenSelect: 
				SelectorToStr := 'kComponentOpenSelect';
			kComponentCloseSelect: 
				SelectorToStr := 'kComponentCloseSelect';
			kComponentTargetSelect: 
				SelectorToStr := 'kComponentTargetSelect';
			kComponentRegisterSelect: 
				SelectorToStr := 'kComponentRegisterSelect';
			kComponentUnregisterSelect: 
				SelectorToStr := 'kComponentUnregisterSelect';
			(* this component type stuff *)
(* Start ICCSelToStr.p *)

			kICCStart: 
			    SelectorToStr := 'kICCStart';
			kICCStop: 
			    SelectorToStr := 'kICCStop';
			kICCGetVersion: 
			    SelectorToStr := 'kICCGetVersion';
			kICCFindConfigFile: 
			    SelectorToStr := 'kICCFindConfigFile';
			kICCFindUserConfigFile: 
			    SelectorToStr := 'kICCFindUserConfigFile';
			kICCGeneralFindConfigFile: 
			    SelectorToStr := 'kICCGeneralFindConfigFile';
			kICCChooseConfig: 
			    SelectorToStr := 'kICCChooseConfig';
			kICCChooseNewConfig: 
			    SelectorToStr := 'kICCChooseNewConfig';
			kICCGetConfigName: 
			    SelectorToStr := 'kICCGetConfigName';
			kICCGetConfigReference: 
			    SelectorToStr := 'kICCGetConfigReference';
			kICCSetConfigReference: 
			    SelectorToStr := 'kICCSetConfigReference';
			kICCSpecifyConfigFile: 
			    SelectorToStr := 'kICCSpecifyConfigFile';
			kICCRefreshCaches: 
			    SelectorToStr := 'kICCRefreshCaches';
			kICCGetSeed: 
			    SelectorToStr := 'kICCGetSeed';
			kICCGetPerm: 
			    SelectorToStr := 'kICCGetPerm';
			kICCDefaultFileName: 
			    SelectorToStr := 'kICCDefaultFileName';
			kICCBegin: 
			    SelectorToStr := 'kICCBegin';
			kICCGetPref: 
			    SelectorToStr := 'kICCGetPref';
			kICCSetPref: 
			    SelectorToStr := 'kICCSetPref';
			kICCFindPrefHandle: 
			    SelectorToStr := 'kICCFindPrefHandle';
			kICCGetPrefHandle: 
			    SelectorToStr := 'kICCGetPrefHandle';
			kICCSetPrefHandle: 
			    SelectorToStr := 'kICCSetPrefHandle';
			kICCCountPref: 
			    SelectorToStr := 'kICCCountPref';
			kICCGetIndPref: 
			    SelectorToStr := 'kICCGetIndPref';
			kICCDeletePref: 
			    SelectorToStr := 'kICCDeletePref';
			kICCEnd: 
			    SelectorToStr := 'kICCEnd';
			kICCGetDefaultPref: 
			    SelectorToStr := 'kICCGetDefaultPref';
			kICCEditPreferences: 
			    SelectorToStr := 'kICCEditPreferences';
			kICCParseURL: 
			    SelectorToStr := 'kICCParseURL';
			kICCLaunchURL: 
			    SelectorToStr := 'kICCLaunchURL';
			kICCMapFilename: 
			    SelectorToStr := 'kICCMapFilename';
			kICCMapTypeCreator: 
			    SelectorToStr := 'kICCMapTypeCreator';
			kICCMapEntriesFilename: 
			    SelectorToStr := 'kICCMapEntriesFilename';
			kICCMapEntriesTypeCreator: 
			    SelectorToStr := 'kICCMapEntriesTypeCreator';
			kICCCountMapEntries: 
			    SelectorToStr := 'kICCCountMapEntries';
			kICCGetIndMapEntry: 
			    SelectorToStr := 'kICCGetIndMapEntry';
			kICCGetMapEntry: 
			    SelectorToStr := 'kICCGetMapEntry';
			kICCSetMapEntry: 
			    SelectorToStr := 'kICCSetMapEntry';
			kICCDeleteMapEntry: 
			    SelectorToStr := 'kICCDeleteMapEntry';
			kICCAddMapEntry: 
			    SelectorToStr := 'kICCAddMapEntry';
			kICCGetCurrentProfile: 
			    SelectorToStr := 'kICCGetCurrentProfile';
			kICCSetCurrentProfile: 
			    SelectorToStr := 'kICCSetCurrentProfile';
			kICCCountProfiles: 
			    SelectorToStr := 'kICCCountProfiles';
			kICCGetIndProfile: 
			    SelectorToStr := 'kICCGetIndProfile';
			kICCGetProfileName: 
			    SelectorToStr := 'kICCGetProfileName';
			kICCSetProfileName: 
			    SelectorToStr := 'kICCSetProfileName';
			kICCAddProfile: 
			    SelectorToStr := 'kICCAddProfile';
			kICCDeleteProfile: 
			    SelectorToStr := 'kICCDeleteProfile';
			kICCRequiresInterruptSafe: 
			    SelectorToStr := 'kICCRequiresInterruptSafe';
			kICCGetMappingInterruptSafe: 
			    SelectorToStr := 'kICCGetMappingInterruptSafe';
			kICCGetSeedInterruptSafe: 
			    SelectorToStr := 'kICCGetSeedInterruptSafe';

(* End ICCSelToStr.p *)
			otherwise
				SelectorToStr := 'other';
		end; (* case *)
	end; (* SelectorToStr *)

	(* ***** Resources and Resource Management ***** *)

	function StartUsingICResources(globals: ICInstanceGlobalsPtr; var didOpen : Boolean) : ICError;
		(* See comment in interface part. *)
		var
			err : ICError;
	begin
		err := noErr;
		if globals^.resourceFileRefNum <> 0 then begin
			didOpen := false;
		end else begin
			globals^.oldResourceFileRefNum := CurResFile;
			
			{$ifc TARGET_API_MAC_CARBON}
				globals^.resourceFileRefNum := FSpOpenResFile(globals^.self, fsRdPerm);
			{$elsec}
				globals^.resourceFileRefNum := OpenComponentResFile(Component(globals^.self));
			{$endc}
			if globals^.resourceFileRefNum <= 0 then begin
				globals^.resourceFileRefNum := 0;
				err := resNotFound;
			end; (* if *)
			didOpen := (err = noErr);
		end; (* if *)
		StartUsingICResources := err;
	end; (* StartUsingICResources *)

	function StopUsingICResources(globals: ICInstanceGlobalsPtr; didOpen : Boolean) : ICError;
		(* See comment in interface part. *)
		var
			err : ICError;
	begin
		err := noErr;
		if didOpen then begin
			{$ifc TARGET_API_MAC_CARBON}
				CloseResFile(globals^.resourceFileRefNum);
				err := ResError;
			{$elsec}
				err := CloseComponentResFile(globals^.resourceFileRefNum);
			{$endc}
			if err = noErr then begin
				globals^.resourceFileRefNum := 0;
				UseResFile(globals^.oldResourceFileRefNum);
			end; (* if *)
		end; (* if *)
		StopUsingICResources := err;
	end; (* StopUsingICResources *)

	function ICGetIndStringResource (globals: ICInstanceGlobalsPtr; 
									strListID: integer; index: integer; str: StringPtr): ICError;
		(* See comment in interface part. *)
		var
			err: ICError;
			junk: ICError;
			didOpen : Boolean;
			tmpStr : Str255;
	begin
		err := StartUsingICResources(globals, didOpen);
		if err = noErr then begin
			GetIndString(tmpStr, strListID, index);
			if tmpStr <> '' then begin
				(* Only copy out the required number of bytes, ie the number
					of bytes that are actually in the string. This is because
					lots of our clients pass in things that aren't Str255's
					(ie Str31s and Str63s), and we don't want to blat
					beyond what they've allocated.
				*)
				BlockMoveData(@tmpStr[0], str, length(tmpStr) + 1);
			end; (* if *)
			junk := StopUsingICResources(globals, didOpen);
		end; (* if *)
		ICGetIndStringResource := err;
	end; (* ICGetIndStringResource *)

end. (* ICExtensionResources *)
