unit ICLibrary;

(* Start Header *)

(* File:         ICLibrary.p
 * Generated by: 1.0d8
 * For:          IC 2.1
 * On:           Monday, 4 October 1999, 0:53:14
 * 
 * This file is part of the Internet Configuration system and
 * is placed in the public domain for the benefit of all.
 *)

(* End Header *)

(*
	This file contains the entry points for the IC shared library-only
	implementation (currently slated for Mac OS X but there's no reason why
	I couldn't use this technology for Mac OS as well).
*)

interface

	uses
		MacTypes,
		CodeFragments,
		
		InternetConfig;

	function  InitICLibrary(const initBlock : CFragInitBlock) : OSErr;
	procedure TermICLibrary;

(*	The following directives ensure that the enclosed symbols are exported
	as CFM entry points.
*)

{$LIBEXPORT+}
{$EXPORT+}

(* Start ICAPI.p *)

	
	(* ***** Starting Up and Shutting Down ***** *)
	
	function ICStart(var inst : ICInstance; creator : OSType) : ICError;
	  (* Call this at application initialisation. Set creator to your application
	   * creator to allow for future expansion of the IC system. Returns
	   * inst as a connection to the IC system.
	   *)
	function ICStop(inst : ICInstance) : ICError;
	  (* [b5] 
	   * Call this at application initialisation, after which inst
	   * is no longer valid connection to IC.
	   *)
	function ICGetVersion(inst : ICInstance; whichVersion : longint; var version : longint) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 8, 50, $7000, $A82A;
	{$endc}
	  (* [r4] [c3] [b3] 
	   * Returns the version of Internet Config.  Pass kICComponentVersion
	   * to get the version as previously returned by GetComponenVerson.
	   * Pass kICNumVersion to get a NumVersion structure.
	   *)
	
	(* ***** Specifying a Configuration ***** *)
	
	{$ifc CALL_NOT_IN_CARBON}
	function ICFindConfigFile(inst : ICInstance; count : integer; folders : ICDirSpecArrayPtr) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 6, 2, $7000, $A82A;
	{$endc}
	  (* [b5] 
	   * Call to configure this connection to IC.
	   * Set count as the number of valid elements in folders.
	   * Set folders to a pointer to the folders to search.
	   * Setting count to 0 and folders to nil is OK.
	   * Searches the specified folders and then the Preferences folder
	   * in a unspecified manner.
	   *)
	{$endc}
	{$ifc CALL_NOT_IN_CARBON}
	function ICFindUserConfigFile(inst : ICInstance; var where : ICDirSpec) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 4, 14, $7000, $A82A;
	{$endc}
	  (* [r1] [b5] 
	   * Similar to ICFindConfigFile except that it only searches the folder
	   * specified in where.  If the input parameters are valid the routine
	   * will always successful configure the instance, creating an
	   * empty configuration if necessary
	   * For use with double-clickable preference files.
	   *)
	{$endc}
	{$ifc CALL_NOT_IN_CARBON}
	function ICGeneralFindConfigFile(inst : ICInstance; searchPrefs : Boolean; canCreate : Boolean; count : integer; folders : ICDirSpecArrayPtr) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 10, 30, $7000, $A82A;
	{$endc}
	  (* [r2] [b5] 
	   * Call to configure this connection to IC.
	   * This routine acts as a more general replacement for
	   * ICFindConfigFile and ICFindUserConfigFile.
	   * Set search_prefs to true if you want it to search the preferences folder.
	   * Set can_create to true if you want it to be able to create a new config.
	   * Set count as the number of valid elements in folders.
	   * Set folders to a pointer to the folders to search.
	   * Setting count to 0 and folders to nil is OK.
	   * Searches the specified folders and then optionally the Preferences folder
	   * in a unspecified manner.
	   *)
	{$endc}
	{$ifc CALL_NOT_IN_CARBON}
	function ICChooseConfig(inst : ICInstance) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 0, 33, $7000, $A82A;
	{$endc}
	  (* [r2] [b5] 
	   * Requests the user to choose a configuration, typically using some
	   * sort of modal dialog. If the user cancels the dialog the configuration
	   * state will be unaffected.
	   *)
	{$endc}
	{$ifc CALL_NOT_IN_CARBON}
	function ICChooseNewConfig(inst : ICInstance) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 0, 34, $7000, $A82A;
	{$endc}
	  (* [r2] [b5] 
	   * Requests the user to create a new configuration, typically using some
	   * sort of modal dialog. If the user cancels the dialog the configuration
	   * state will be unaffected.
	   *)
	{$endc}
	function ICGetConfigName(inst : ICInstance; longname : Boolean; var name : Str255) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 6, 35, $7000, $A82A;
	{$endc}
	  (* [r2] [c1] [b3] 
	   * Returns a string that describes the current configuration at a user
	   * level. Set longname to true if you want a long name, up to 255
	   * characters, or false if you want a short name, typically about 32
	   * characters.
	   * The returned string is for user display only. If you rely on the
	   * exact format of it, you will conflict with any future IC
	   * implementation that doesn't use explicit preference files.
	   *)
	{$ifc CALL_NOT_IN_CARBON}
	function ICGetConfigReference(inst : ICInstance; ref : ICConfigRefHandle) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 4, 31, $7000, $A82A;
	{$endc}
	  (* [r2] [c1] [b3] 
	   * Returns a self-contained reference to the instance's current
	   * configuration.
	   * ref must be a valid non-nil handle and it will be resized to fit the
	   * resulting data.
	   *)
	{$endc}
	{$ifc CALL_NOT_IN_CARBON}
	function ICSetConfigReference(inst : ICInstance; ref : ICConfigRefHandle; flags : longint) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 8, 32, $7000, $A82A;
	{$endc}
	  (* [r2] [b5] 
	   * Reconfigures the instance using a configuration reference that was
	   * got using ICGetConfigReference reference. Set the
	   * icNoUserInteraction_bit in flags if you require that this routine
	   * not present a modal dialog. Other flag bits are reserved and should
	   * be set to zero.
	   * ref must not be nil.
	   *)
	{$endc}
	
	(* ***** Private Routines *****
	 * 
	 * If you are calling these routines, you are most probably doing something
	 * wrong.  Please read the documentation for more details.
	 *)
	
	{$ifc CALL_NOT_IN_CARBON}
	function ICSpecifyConfigFile(inst : ICInstance; var config : FSSpec) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 4, 3, $7000, $A82A;
	{$endc}
	  (* [b5] 
	   * For use only by the IC application.
	   * If you call this routine yourself, you will conflict with any
	   * future IC implementation that doesn't use explicit preference files.
	   *)
	{$endc}
	{$ifc CALL_NOT_IN_CARBON}
	function ICRefreshCaches(inst : ICInstance) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 0, 47, $7000, $A82A;
	{$endc}
	  (* [r3] [c1] [b3] 
	   * For use only by the IC application.
	   * If you call this routine yourself, you will conflict with any
	   * future IC implementation that doesn't use explicit preference files.
	   *)
	{$endc}
	
	(* ***** Getting Information ***** *)
	
	function ICGetSeed(inst : ICInstance; var seed : longint) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 4, 4, $7000, $A82A;
	{$endc}
	  (* [c3] [b3] 
	   * Returns the current seed for the IC prefs database.
	   * This seed changes each time a non-volatile preference is changed.
	   * You can poll this to determine if any cached preferences change.
	   *)
	function ICGetPerm(inst : ICInstance; var perm : ICPerm) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 4, 13, $7000, $A82A;
	{$endc}
	  (* [c3] [b3] 
	   * Returns the access permissions currently associated with this instance.
	   * While applications normally know what permissions they have,
	   * this routine is designed for use by override components.
	   *)
	{$ifc CALL_NOT_IN_CARBON}
	function ICDefaultFileName(inst : ICInstance; var name : Str63) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 4, 11, $7000, $A82A;
	{$endc}
	  (* [c3] [b3] 
	   * Returns the default file name for IC preference files.
	   * Applications should never need to call this routine.
	   * If you rely on information returned by this routine yourself,
	   * you may conflict with any future IC implementation that doesn't use
	   * explicit preference files.
	   * The component calls this routine to set up the default IC file name.
	   * This allows this operation to be intercepted by a component that has
	   * captured us. It currently gets it from the component resource file.
	   * The glue version is hardwired to "Internet Preferences".
	   *)
	{$endc}
	{$ifc CALL_NOT_IN_CARBON}
	function ICGetComponentInstance(inst : ICInstance; var componentInst : ComponentInstance) : ICError;
	  (* [r4] [c3] [b3] 
	   * Returns noErr and the connection to the IC component,
	   * if we're using the component.
	   * Returns badComponenInstance and nil if we're operating with glue.
	   *)
	{$endc}
	
	(* ***** Reading and Writing Preferences ***** *)
	
	function ICBegin(inst : ICInstance; perm : ICPerm) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 2, 5, $7000, $A82A;
	{$endc}
	  (* [c1] [b5] 
	   * Starting reading or writing multiple preferences.
	   * A call to this must be balanced by a call to ICEnd.
	   * Do not call WaitNextEvent between these calls.
	   * The perm specifies whether you intend to read or read/write.
	   * Only one writer is allowed per instance.
	   * Note that this may open resource files that are not closed
	   * until you call ICEnd.
	   *)
	function ICGetPref(inst : ICInstance; key : Str255; var attr : ICAttr; buf : Ptr; var size : longint) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 16, 6, $7000, $A82A;
	{$endc}
	  (* [c1] [b4] 
	   * Reads the preference specified by key from the IC database to the
	   * buffer pointed to by buf and size.
	   * key must not be the empty string.
	   * If buf is nil then no data is returned.
	   * size must be non-negative.
	   * attr and size are always set on return. On errors (except icTruncatedErr)
	   * attr is set to ICattr_no_change and size is set to 0.
	   * size is the actual size of the data.
	   * attr is set to the attributes associated with the preference.
	   * If this routine returns icTruncatedErr then the other returned
	   * values are valid except that only the first size bytes have been
	   * return. size is adjusted to reflect the true size of the preference.
	   * Returns icPrefNotFound if there is no preference for the key.
	   *)
	function ICSetPref(inst : ICInstance; key : Str255; attr : ICAttr; buf : Ptr; size : longint) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 16, 7, $7000, $A82A;
	{$endc}
	  (* [c1] [b4] 
	   * Sets the preference specified by key from the IC database to the
	   * value pointed to by buf and size.
	   * key must not be the empty string.
	   * size must be non-negative. 
	   * If buf is nil then the preference value is not set and size is ignored.
	   * If buf is not nil then the preference value is set to the size
	   * bytes pointed to by buf.
	   * If attr is ICattr_no_change then the preference attributes are not set.
	   * Otherwise the preference attributes are set to attr.
	   * Returns icPermErr if the previous ICBegin was passed icReadOnlyPerm.
	   * Returns icPermErr if current attr is locked, new attr is locked and buf <> nil.
	   *)
	function ICFindPrefHandle(inst : ICInstance; key : Str255; var attr : ICAttr; prefh : Handle) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 12, 36, $7000, $A82A;
	{$endc}
	  (* [r2] [c1] [b4] 
	   * This routine effectively replaces ICGetPrefHandle.
	   * Reads the preference specified by key from the IC database into
	   * a handle, prefh.
	   * key must not be the empty string.
	   * attr is set to the attributes associated with the preference.
	   * You must set prefh to a non-nil handle before calling this routine.
	   * If the preference does not exist, icPrefNotFoundErr is returned.
	   *)
	function ICGetPrefHandle(inst : ICInstance; key : Str255; var attr : ICAttr; var prefh : Handle) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 12, 26, $7000, $A82A;
	{$endc}
	  (* [r1] [c1] [b4] 
	   * This routine is now obsolete. Use ICFindPrefHandle instead.
	   * Reads the preference specified by key from the IC database into
	   * a newly created handle, prefh.
	   * key must not be the empty string.
	   * attr is set to the attributes associated with the preference.
	   * The incoming value of prefh is ignored.
	   * A new handle is created in the current heap and returned in prefh.
	   * If the routine returns an error, prefh is set to nil.
	   * If the preference does not exist, no error is returned and prefh is set
	   * to an empty handle.
	   *)
	function ICSetPrefHandle(inst : ICInstance; key : Str255; attr : ICAttr; prefh : Handle) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 12, 27, $7000, $A82A;
	{$endc}
	  (* [r1] [c1] [b4] 
	   * Sets the preference specified by key from the IC database to the
	   * value contained in prefh.
	   * key must not be the empty string.
	   * If prefh is nil then the preference value is not set.
	   * If buf is not nil then the preference value is set to the data
	   * contained in it.
	   * If attr is ICattr_no_change then the preference attributes are not set.
	   * Otherwise the preference attributes are set to attr.
	   * Returns icPermErr if the previous ICBegin was passed icReadOnlyPerm.
	   * Returns icPermErr if current attr is locked, new attr is locked and prefh <> nil.
	   *)
	function ICCountPref(inst : ICInstance; var count : longint) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 4, 8, $7000, $A82A;
	{$endc}
	  (* [c1] [b1] 
	   * Counts the total number of preferences.
	   * If the routine returns an error, count is set to 0.
	   *)
	function ICGetIndPref(inst : ICInstance; index : longint; var key : Str255) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 8, 9, $7000, $A82A;
	{$endc}
	  (* [c1] [b1] 
	   * Returns the key of the index'th preference.
	   * index must be positive.
	   * Returns icPrefNotFoundErr if index is greater than the total number of preferences.
	   * If the routine returns an error, key is undefined.
	   *)
	function ICDeletePref(inst : ICInstance; key : Str255) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 4, 12, $7000, $A82A;
	{$endc}
	  (* [c1] [b2] 
	   * Deletes the preference specified by key.
	   * key must not be the empty string.
	   * Returns icPrefNotFound if the preference specified by key is not present.
	   *)
	function ICEnd(inst : ICInstance) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 0, 10, $7000, $A82A;
	{$endc}
	  (* [c1] [b1] 
	   * Terminates a preference session, as started by ICBegin.
	   * You must have called ICBegin before calling this routine.
	   *)
	function ICGetDefaultPref(inst : ICInstance; key : Str255; prefH : Handle) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 8, 49, $7000, $A82A;
	{$endc}
	  (* [r4] [c3] [b5] 
	   * Returns a default preference value for the specified key.  You
	   * must pass in a valid prefH, which is resized to fit the data.
	   *)
	
	(* ***** User Interface Stuff ***** *)
	
	function ICEditPreferences(inst : ICInstance; key : Str255) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 4, 15, $7000, $A82A;
	{$endc}
	  (* [r1] [c1] [b3] 
	   * Instructs IC to display the user interface associated with editing
	   * preferences and focusing on the preference specified by key.
	   * If key is the empty string then no preference should be focused upon.
	   * You must have specified a configuration before calling this routine.
	   * You do not need to call ICBegin before calling this routine.
	   * In the current implementation this launches the IC application
	   * (or brings it to the front) and displays the window containing
	   * the preference specified by key.
	   * It may have a radically different implementation in future
	   * IC systems.
	   *)
	
	(* ***** URL Handling ***** *)
	
	function ICParseURL(inst : ICInstance; hint : Str255; data : Ptr; len : longint; var selStart : longint; var selEnd : longint; url : Handle) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 24, 16, $7000, $A82A;
	{$endc}
	  (* [r1] [c1] [b3] 
	   * Parses a URL out of the specified text and returns it in a canonical form
	   * in a handle.
	   * hint indicates the default scheme for URLs of the form "name@address".
	   * If hint is the empty string then URLs of that form are not allowed.
	   * data points to the start of the text. It must not be nil.
	   * len indicates the length of the text. It must be non-negative.
	   * selStart and selEnd should be passed in as the current selection of
	   * the text. This selection is given in the same manner as TextEdit,
	   * ie if selStart = selEnd then there is no selection only an insertion
	   * point. Also selStart  selEnd and 0  selStart  len and 0  selEnd  len.
	   * selStart and selEnd are returned as the bounds of the URL. If the
	   * routine returns an error then these new boundaries may be
	   * invalid but they will be close.
	   * The incoming url handle must not be nil.  The resulting URL is normalised
	   * and copied into the url handle, which is resized to fit.
	   *)
	function ICLaunchURL(inst : ICInstance; hint : Str255; data : Ptr; len : longint; var selStart : longint; var selEnd : longint) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 20, 17, $7000, $A82A;
	{$endc}
	  (* [r1] [c1] [b3] 
	   * Parses a URL out of the specified text and feeds it off to the
	   * appropriate helper.
	   * hint indicates the default scheme for URLs of the form "name@address".
	   * If hint is the empty string then URLs of that form are not allowed.
	   * data points to the start of the text. It must not be nil.
	   * len indicates the length of the text. It must be non-negative.
	   * selStart and selEnd should be passed in as the current selection of
	   * the text. This selection is given in the same manner as TextEdit,
	   * ie if selStart = selEnd then there is no selection only an insertion
	   * point. Also selStart  selEnd and 0  selStart  len and 0  selEnd  len.
	   * selStart and selEnd are returned as the bounds of the URL. If the
	   * routine returns an error then these new boundaries may be
	   * invalid but they will be close.
	   * The URL is parsed out of the text and passed off to the appropriate
	   * helper using the GURL AppleEvent.
	   *)
	
	(* ***** Mappings Routines *****
	 * 
	 * Routines for interrogating mappings database.
	 * 
	 * ----- High Level Routines -----
	 *)
	
	function ICMapFilename(inst : ICInstance; filename : Str255; var entry : ICMapEntry) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 8, 24, $7000, $A82A;
	{$endc}
	  (* [r1] [c1] [b4] 
	   * Takes the name of an incoming file and returns the most appropriate
	   * mappings database entry, based on its extension.
	   * filename must not be the empty string.
	   * Returns icPrefNotFoundErr if no suitable entry is found.
	   *)
	function ICMapTypeCreator(inst : ICInstance; fType : OSType; fCreator : OSType; filename : Str255; var entry : ICMapEntry) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 16, 25, $7000, $A82A;
	{$endc}
	  (* [r1] [c1] [b4] 
	   * Takes the type and creator (and optionally the name) of an outgoing
	   * file and returns the most appropriate mappings database entry.
	   * The filename may be either the name of the outgoing file or
	   * the empty string.
	   * Returns icPrefNotFoundErr if no suitable entry found.
	   *)
	
	(* ----- Mid Level Routines ----- *)
	
	function ICMapEntriesFilename(inst : ICInstance; entries : Handle; filename : Str255; var entry : ICMapEntry) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 12, 28, $7000, $A82A;
	{$endc}
	  (* [r1] [c1] [b3] 
	   * Takes the name of an incoming file and returns the most appropriate
	   * mappings database entry, based on its extension.
	   * entries must be a handle to a valid IC mappings database preference.
	   * filename must not be the empty string.
	   * Returns icPrefNotFoundErr if no suitable entry is found.
	   *)
	function ICMapEntriesTypeCreator(inst : ICInstance; entries : Handle; fType : OSType; fCreator : OSType; filename : Str255; var entry : ICMapEntry) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 20, 29, $7000, $A82A;
	{$endc}
	  (* [r1] [c1] [b3] 
	   * Takes the type and creator (and optionally the name) of an outgoing
	   * file and returns the most appropriate mappings database entry.
	   * entries must be a handle to a valid IC mappings database preference.
	   * The filename may be either the name of the outgoing file or
	   * the empty string.
	   * Returns icPrefNotFoundErr if no suitable entry found.
	   *)
	
	(* ----- Low Level Routines ----- *)
	
	function ICCountMapEntries(inst : ICInstance; entries : Handle; var count : longint) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 8, 18, $7000, $A82A;
	{$endc}
	  (* [r1] [c1] [b3] 
	   * Counts the number of entries in the mappings database.
	   * entries must be a handle to a valid IC mappings database preference.
	   * count is set to the number of entries.
	   *)
	function ICGetIndMapEntry(inst : ICInstance; entries : Handle; index : longint; var pos : longint; var entry : ICMapEntry) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 16, 19, $7000, $A82A;
	{$endc}
	  (* [r1] [c1] [b3] 
	   * Gets the index'th entry in the mappings database.
	   * entries must be a handle to a valid IC mappings database preference.
	   * index must be in the range from 1 to the number of entries in the database.
	   * The value of pos is ignored on input. pos is set to the position of
	   * the index'th entry in the database and is suitable for passing back
	   * into ICSetMapEntry.
	   * Does not return any user data associated with the entry.
	   *)
	function ICGetMapEntry(inst : ICInstance; entries : Handle; pos : longint; var entry : ICMapEntry) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 12, 20, $7000, $A82A;
	{$endc}
	  (* [r1] [c1] [b3] 
	   * Returns the entry located at position pos in the mappings database.
	   * entries must be a handle to a valid IC mappings database preference.
	   * pos should be 0 to get the first entry. To get the subsequent entries, add
	   * entry.total_size to pos and iterate.
	   * Does not return any user data associated with the entry.
	   *)
	function ICSetMapEntry(inst : ICInstance; entries : Handle; pos : longint; var entry : ICMapEntry) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 12, 21, $7000, $A82A;
	{$endc}
	  (* [r1] [c1] [b3] 
	   * Sets the entry located at position pos in the mappings database.
	   * entries must be a handle to a valid IC mappings database preference.
	   * pos should be either a value returned from ICGetIndMapEntry or a value
	   * calculated using ICGetMapEntry.
	   * entry is a var parameter purely for stack space reasons. It is not
	   * modified in any way.
	   * Any user data associated with the entry is unmodified.
	   *)
	function ICDeleteMapEntry(inst : ICInstance; entries : Handle; pos : longint) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 8, 22, $7000, $A82A;
	{$endc}
	  (* [r1] [c1] [b3] 
	   * Deletes the mappings database entry at pos.
	   * entries must be a handle to a valid IC mappings database preference.
	   * pos should be either a value returned from ICGetIndMapEntry or a value
	   * calculated using ICGetMapEntry.
	   * Also deletes any user data associated with the entry.
	   *)
	function ICAddMapEntry(inst : ICInstance; entries : Handle; var entry : ICMapEntry) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 8, 23, $7000, $A82A;
	{$endc}
	  (* [r1] [c1] [b3] 
	   * Adds an entry to the mappings database.
	   * entries must be a handle to a valid IC mappings database preference.
	   * The entry is added to the end of the entries database.
	   * No user data is added.
	   *)
	
	(* ***** Profile Management Routines ***** *)
	
	function ICGetCurrentProfile(inst : ICInstance; var currentID : ICProfileID) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 4, 37, $7000, $A82A;
	{$endc}
	  (* [r3] [c1] [b3] 
	   * Returns the profile ID of the current profile.
	   *)
	function ICSetCurrentProfile(inst : ICInstance; newID : ICProfileID) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 4, 38, $7000, $A82A;
	{$endc}
	  (* [r3] [c1] [b3] 
	   * Sets the current profile to the profile specified in newProfile.
	   *)
	function ICCountProfiles(inst : ICInstance; var count : longint) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 4, 39, $7000, $A82A;
	{$endc}
	  (* [r3] [c1] [b1] 
	   * Returns the total number of profiles.
	   *)
	function ICGetIndProfile(inst : ICInstance; index : longint; var thisID : ICProfileID) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 8, 40, $7000, $A82A;
	{$endc}
	  (* [r3] [c1] [b1] 
	   * Returns the profile ID of the index'th profile.  index must be positive.
	   * Returns icProfileNotFoundErr if index is greater than the total number
	   * of profiles.
	   *)
	function ICGetProfileName(inst : ICInstance; thisID : ICProfileID; var name : Str255) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 8, 41, $7000, $A82A;
	{$endc}
	  (* [r3] [c1] [b3] 
	   * Returns the name of a profile given its ID.  The name may not uniquely
	   * identify the profile.  [That's what the profile ID is for!]  The name
	   * is assumed to be in the system script.
	   *)
	function ICSetProfileName(inst : ICInstance; thisID : ICProfileID; name : Str255) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 8, 42, $7000, $A82A;
	{$endc}
	  (* [r3] [c1] [b3] 
	   * This routine sets the name of the specified profile.  Profile names
	   * need not be unique.  The name should be in the system script.
	   *)
	function ICAddProfile(inst : ICInstance; prototypeID : ICProfileID; var newID : ICProfileID) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 8, 43, $7000, $A82A;
	{$endc}
	  (* [r3] [c1] [b2] 
	   * If prototypeID = kICNilProfileID, this routine
	   * creates a default profile, otherwise it creates a
	   * profile by cloning the prototype profile.  The ID
	   * of the new profile is returned in newID.
	   * The new profile will be give a new, unique, name.
	   * This does not switch to the new profile.
	   *)
	function ICDeleteProfile(inst : ICInstance; thisID : ICProfileID) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 4, 44, $7000, $A82A;
	{$endc}
	  (* [r3] [c1] [b2] 
	   * This routine deletes the profile specified by
	   * thisID.  Attempting to delete the current profile
	   * or the last profile will return error.
	   *)
	
	(* ***** Interrupt Safe Routines ***** *)
	
	function ICRequiresInterruptSafe(inst : ICInstance) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 0, 45, $7000, $A82A;
	{$endc}
	  (* [r3] [c2] [b3] 
	   * You must call this routine before calling GetMapEntryInterruptSafe
	   * to give IC chance to cache the mappings data in memory.  The only
	   * way to clear this state is to close the instance.  You can not reconfigure
	   * the instance after calling this routine.
	   *)
	function ICGetMappingInterruptSafe(inst : ICInstance; var mappingPref : Ptr; var mappingPrefSize : longint) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 8, 46, $7000, $A82A;
	{$endc}
	  (* [r3] [c2] [b3] 
	   * Returns the "Mapping" preference in an interrupt safe fashion.
	   * The preference returned pointer is valid until the next
	   * non-interrupt safe call to IC.  Typically this API is used
	   * by software that needs to map extensions to type and creator
	   * at interrupt time, eg foreign file systems.
	   *)
	function ICGetSeedInterruptSafe(inst : ICInstance; var seed : longint) : ICError;
	{$ifc TARGET_OS_MAC AND TARGET_CPU_68K AND NOT TARGET_RT_MAC_CFM}
	  inline $2F3C, 4, 48, $7000, $A82A;
	{$endc}
	  (* [r3] [c2] [b3] 
	   * An interrupt safe version of ICGetSeed.
	   *)

(* End ICAPI.p *)

{$EXPORT-}
{$LIBEXPORT-}

implementation

	uses
		CodeFragments,
		
		ICDebug,
		ICCommonSubs,
		ICExtensionGlobals,
		ICCore,
		ICURLHandling,
		ICMapping,
		ICProfiles,
		ICMultiUser;
		
	var
		gLibraryFSSpec : FSSpec;

	function  InitICLibrary(const initBlock : CFragInitBlock): OSErr;
		var
			err : OSStatus;
			junk : OSStatus;
	begin
		if initBlock.fragLocator.where = kDataForkCFragLocator then begin
			gLibraryFSSpec := initBlock.fragLocator.onDisk.fileSpec^;
			err := noErr;
		end else begin
			err := unimpErr;
			ICDebugStr('InitICLibrary: Library init routine failed');
		end; (* if *)
		
		// Not being able to hook up the multi-user callback routine
		// is not a sufficently important reason to cause the entire
		// closure to not load, which is what happens if you return
		// an error from your CFM init proc.  So we just swallow the
		// error on production builds.
		
		junk := InitICMultiUser;
		ICAssert(junk = noErr);

		InitICLibrary := err;
	end; (* InitICLibrary *)
	
	procedure TermICLibrary;
	begin
		TermICMultiUser;
	end; (* TermICLibrary *)
	
	function ICStart(var inst : ICInstance; creator : OSType) : ICError;
		var
			err : OSStatus;
			globals : ICInstanceGlobalsPtr;
	begin
		globals := ICInstanceGlobalsPtr(NewPtrClear(sizeof(ICInstanceGlobals)));
		err := CheckMemError(globals);
		inst := ICInstance(globals);

		if err = noErr then begin
			globals^.self := gLibraryFSSpec;
			globals^.target := inst;
			err := ICCIStart(globals, creator);
		end; (* if *)
		ICStart := err;
	end; (* ICStart *)
	
	function ICStop(inst : ICInstance) : ICError;
		var
			err : OSStatus;
	begin
		err := ICCIStop(ICInstanceGlobalsPtr(inst));
		if err = noErr then begin
			DisposePtr(Ptr(inst));
			ICAssert(MemError = noErr);
		end; (* if *)
		ICStop := err;
	end; (* ICStop *)
	
(* Start ICLibrary.p *)

	(* ***** Starting Up and Shutting Down ***** *)
	
	function ICGetVersion(inst : ICInstance; whichVersion : longint; var version : longint) : ICError;
	begin
	    ICGetVersion := ICCIGetVersion(ICInstanceGlobalsPtr(inst), whichVersion, version);
	end; (* ICGetVersion *)
	
	(* ***** Specifying a Configuration ***** *)
	
	{$ifc CALL_NOT_IN_CARBON}
	function ICFindConfigFile(inst : ICInstance; count : integer; folders : ICDirSpecArrayPtr) : ICError;
	begin
	    ICFindConfigFile := ICCIFindConfigFile(ICInstanceGlobalsPtr(inst), count, folders);
	end; (* ICFindConfigFile *)
	{$endc}
	
	{$ifc CALL_NOT_IN_CARBON}
	function ICFindUserConfigFile(inst : ICInstance; var where : ICDirSpec) : ICError;
	begin
	    ICFindUserConfigFile := ICCIFindUserConfigFile(ICInstanceGlobalsPtr(inst), where);
	end; (* ICFindUserConfigFile *)
	{$endc}
	
	{$ifc CALL_NOT_IN_CARBON}
	function ICGeneralFindConfigFile(inst : ICInstance; searchPrefs : Boolean; canCreate : Boolean; count : integer; folders : ICDirSpecArrayPtr) : ICError;
	begin
	    ICGeneralFindConfigFile := ICCIGeneralFindConfigFile(ICInstanceGlobalsPtr(inst), searchPrefs, canCreate, count, folders);
	end; (* ICGeneralFindConfigFile *)
	{$endc}
	
	{$ifc CALL_NOT_IN_CARBON}
	function ICChooseConfig(inst : ICInstance) : ICError;
	begin
	    ICChooseConfig := ICCIChooseConfig(ICInstanceGlobalsPtr(inst));
	end; (* ICChooseConfig *)
	{$endc}
	
	{$ifc CALL_NOT_IN_CARBON}
	function ICChooseNewConfig(inst : ICInstance) : ICError;
	begin
	    ICChooseNewConfig := ICCIChooseNewConfig(ICInstanceGlobalsPtr(inst));
	end; (* ICChooseNewConfig *)
	{$endc}
	
	function ICGetConfigName(inst : ICInstance; longname : Boolean; var name : Str255) : ICError;
	begin
	    ICGetConfigName := ICCIGetConfigName(ICInstanceGlobalsPtr(inst), longname, name);
	end; (* ICGetConfigName *)
	
	{$ifc CALL_NOT_IN_CARBON}
	function ICGetConfigReference(inst : ICInstance; ref : ICConfigRefHandle) : ICError;
	begin
	    ICGetConfigReference := ICCIGetConfigReference(ICInstanceGlobalsPtr(inst), ref);
	end; (* ICGetConfigReference *)
	{$endc}
	
	{$ifc CALL_NOT_IN_CARBON}
	function ICSetConfigReference(inst : ICInstance; ref : ICConfigRefHandle; flags : longint) : ICError;
	begin
	    ICSetConfigReference := ICCISetConfigReference(ICInstanceGlobalsPtr(inst), ref, flags);
	end; (* ICSetConfigReference *)
	{$endc}
	
	(* ***** Private Routines *****
	 * 
	 * If you are calling these routines, you are most probably doing something
	 * wrong.  Please read the documentation for more details.
	 *)
	
	{$ifc CALL_NOT_IN_CARBON}
	function ICSpecifyConfigFile(inst : ICInstance; var config : FSSpec) : ICError;
	begin
	    ICSpecifyConfigFile := ICCISpecifyConfigFile(ICInstanceGlobalsPtr(inst), config);
	end; (* ICSpecifyConfigFile *)
	{$endc}
	
	{$ifc CALL_NOT_IN_CARBON}
	function ICRefreshCaches(inst : ICInstance) : ICError;
	begin
	    ICRefreshCaches := ICCIRefreshCaches(ICInstanceGlobalsPtr(inst));
	end; (* ICRefreshCaches *)
	{$endc}
	
	(* ***** Getting Information ***** *)
	
	function ICGetSeed(inst : ICInstance; var seed : longint) : ICError;
	begin
	    ICGetSeed := ICCIGetSeed(ICInstanceGlobalsPtr(inst), seed);
	end; (* ICGetSeed *)
	
	function ICGetPerm(inst : ICInstance; var perm : ICPerm) : ICError;
	begin
	    ICGetPerm := ICCIGetPerm(ICInstanceGlobalsPtr(inst), perm);
	end; (* ICGetPerm *)
	
	{$ifc CALL_NOT_IN_CARBON}
	function ICDefaultFileName(inst : ICInstance; var name : Str63) : ICError;
	begin
	    ICDefaultFileName := ICCIDefaultFileName(ICInstanceGlobalsPtr(inst), name);
	end; (* ICDefaultFileName *)
	{$endc}
	
	{$ifc CALL_NOT_IN_CARBON}
	function ICGetComponentInstance(inst : ICInstance; var componentInst : ComponentInstance) : ICError;
	begin
	    ICGetComponentInstance := ICCIGetComponentInstance(ICInstanceGlobalsPtr(inst), componentInst);
	end; (* ICGetComponentInstance *)
	{$endc}
	
	(* ***** Reading and Writing Preferences ***** *)
	
	function ICBegin(inst : ICInstance; perm : ICPerm) : ICError;
	begin
	    ICBegin := ICCIBegin(ICInstanceGlobalsPtr(inst), perm);
	end; (* ICBegin *)
	
	function ICGetPref(inst : ICInstance; key : Str255; var attr : ICAttr; buf : Ptr; var size : longint) : ICError;
	begin
	    ICGetPref := ICCIGetPref(ICInstanceGlobalsPtr(inst), @key, attr, buf, size);
	end; (* ICGetPref *)
	
	function ICSetPref(inst : ICInstance; key : Str255; attr : ICAttr; buf : Ptr; size : longint) : ICError;
	begin
	    ICSetPref := ICCISetPref(ICInstanceGlobalsPtr(inst), @key, attr, buf, size);
	end; (* ICSetPref *)
	
	function ICFindPrefHandle(inst : ICInstance; key : Str255; var attr : ICAttr; prefh : Handle) : ICError;
	begin
	    ICFindPrefHandle := ICCIFindPrefHandle(ICInstanceGlobalsPtr(inst), @key, attr, prefh);
	end; (* ICFindPrefHandle *)
	
	function ICGetPrefHandle(inst : ICInstance; key : Str255; var attr : ICAttr; var prefh : Handle) : ICError;
	begin
	    ICGetPrefHandle := ICCIGetPrefHandle(ICInstanceGlobalsPtr(inst), @key, attr, prefh);
	end; (* ICGetPrefHandle *)
	
	function ICSetPrefHandle(inst : ICInstance; key : Str255; attr : ICAttr; prefh : Handle) : ICError;
	begin
	    ICSetPrefHandle := ICCISetPrefHandle(ICInstanceGlobalsPtr(inst), @key, attr, prefh);
	end; (* ICSetPrefHandle *)
	
	function ICCountPref(inst : ICInstance; var count : longint) : ICError;
	begin
	    ICCountPref := ICCICountPref(ICInstanceGlobalsPtr(inst), count);
	end; (* ICCountPref *)
	
	function ICGetIndPref(inst : ICInstance; index : longint; var key : Str255) : ICError;
	begin
	    ICGetIndPref := ICCIGetIndPref(ICInstanceGlobalsPtr(inst), index, key);
	end; (* ICGetIndPref *)
	
	function ICDeletePref(inst : ICInstance; key : Str255) : ICError;
	begin
	    ICDeletePref := ICCIDeletePref(ICInstanceGlobalsPtr(inst), @key);
	end; (* ICDeletePref *)
	
	function ICEnd(inst : ICInstance) : ICError;
	begin
	    ICEnd := ICCIEnd(ICInstanceGlobalsPtr(inst));
	end; (* ICEnd *)
	
	function ICGetDefaultPref(inst : ICInstance; key : Str255; prefH : Handle) : ICError;
	begin
	    ICGetDefaultPref := ICCIGetDefaultPref(ICInstanceGlobalsPtr(inst), @key, prefH);
	end; (* ICGetDefaultPref *)
	
	(* ***** User Interface Stuff ***** *)
	
	function ICEditPreferences(inst : ICInstance; key : Str255) : ICError;
	begin
	    ICEditPreferences := ICCIEditPreferences(ICInstanceGlobalsPtr(inst), @key);
	end; (* ICEditPreferences *)
	
	(* ***** URL Handling ***** *)
	
	function ICParseURL(inst : ICInstance; hint : Str255; data : Ptr; len : longint; var selStart : longint; var selEnd : longint; url : Handle) : ICError;
	begin
	    ICParseURL := ICCIParseURL(ICInstanceGlobalsPtr(inst), @hint, data, len, selStart, selEnd, url);
	end; (* ICParseURL *)
	
	function ICLaunchURL(inst : ICInstance; hint : Str255; data : Ptr; len : longint; var selStart : longint; var selEnd : longint) : ICError;
	begin
	    ICLaunchURL := ICCILaunchURL(ICInstanceGlobalsPtr(inst), @hint, data, len, selStart, selEnd);
	end; (* ICLaunchURL *)
	
	(* ***** Mappings Routines *****
	 * 
	 * Routines for interrogating mappings database.
	 * 
	 * ----- High Level Routines -----
	 *)
	
	function ICMapFilename(inst : ICInstance; filename : Str255; var entry : ICMapEntry) : ICError;
	begin
	    ICMapFilename := ICCIMapFilename(ICInstanceGlobalsPtr(inst), @filename, entry);
	end; (* ICMapFilename *)
	
	function ICMapTypeCreator(inst : ICInstance; fType : OSType; fCreator : OSType; filename : Str255; var entry : ICMapEntry) : ICError;
	begin
	    ICMapTypeCreator := ICCIMapTypeCreator(ICInstanceGlobalsPtr(inst), fType, fCreator, @filename, entry);
	end; (* ICMapTypeCreator *)
	
	(* ----- Mid Level Routines ----- *)
	
	function ICMapEntriesFilename(inst : ICInstance; entries : Handle; filename : Str255; var entry : ICMapEntry) : ICError;
	begin
	    ICMapEntriesFilename := ICCIMapEntriesFilename(ICInstanceGlobalsPtr(inst), entries, @filename, entry);
	end; (* ICMapEntriesFilename *)
	
	function ICMapEntriesTypeCreator(inst : ICInstance; entries : Handle; fType : OSType; fCreator : OSType; filename : Str255; var entry : ICMapEntry) : ICError;
	begin
	    ICMapEntriesTypeCreator := ICCIMapEntriesTypeCreator(ICInstanceGlobalsPtr(inst), entries, fType, fCreator, @filename, entry);
	end; (* ICMapEntriesTypeCreator *)
	
	(* ----- Low Level Routines ----- *)
	
	function ICCountMapEntries(inst : ICInstance; entries : Handle; var count : longint) : ICError;
	begin
	    ICCountMapEntries := ICCICountMapEntries(ICInstanceGlobalsPtr(inst), entries, count);
	end; (* ICCountMapEntries *)
	
	function ICGetIndMapEntry(inst : ICInstance; entries : Handle; index : longint; var pos : longint; var entry : ICMapEntry) : ICError;
	begin
	    ICGetIndMapEntry := ICCIGetIndMapEntry(ICInstanceGlobalsPtr(inst), entries, index, pos, entry);
	end; (* ICGetIndMapEntry *)
	
	function ICGetMapEntry(inst : ICInstance; entries : Handle; pos : longint; var entry : ICMapEntry) : ICError;
	begin
	    ICGetMapEntry := ICCIGetMapEntry(ICInstanceGlobalsPtr(inst), entries, pos, entry);
	end; (* ICGetMapEntry *)
	
	function ICSetMapEntry(inst : ICInstance; entries : Handle; pos : longint; var entry : ICMapEntry) : ICError;
	begin
	    ICSetMapEntry := ICCISetMapEntry(ICInstanceGlobalsPtr(inst), entries, pos, entry);
	end; (* ICSetMapEntry *)
	
	function ICDeleteMapEntry(inst : ICInstance; entries : Handle; pos : longint) : ICError;
	begin
	    ICDeleteMapEntry := ICCIDeleteMapEntry(ICInstanceGlobalsPtr(inst), entries, pos);
	end; (* ICDeleteMapEntry *)
	
	function ICAddMapEntry(inst : ICInstance; entries : Handle; var entry : ICMapEntry) : ICError;
	begin
	    ICAddMapEntry := ICCIAddMapEntry(ICInstanceGlobalsPtr(inst), entries, entry);
	end; (* ICAddMapEntry *)
	
	(* ***** Profile Management Routines ***** *)
	
	function ICGetCurrentProfile(inst : ICInstance; var currentID : ICProfileID) : ICError;
	begin
	    ICGetCurrentProfile := ICCIGetCurrentProfile(ICInstanceGlobalsPtr(inst), currentID);
	end; (* ICGetCurrentProfile *)
	
	function ICSetCurrentProfile(inst : ICInstance; newID : ICProfileID) : ICError;
	begin
	    ICSetCurrentProfile := ICCISetCurrentProfile(ICInstanceGlobalsPtr(inst), newID);
	end; (* ICSetCurrentProfile *)
	
	function ICCountProfiles(inst : ICInstance; var count : longint) : ICError;
	begin
	    ICCountProfiles := ICCICountProfiles(ICInstanceGlobalsPtr(inst), count);
	end; (* ICCountProfiles *)
	
	function ICGetIndProfile(inst : ICInstance; index : longint; var thisID : ICProfileID) : ICError;
	begin
	    ICGetIndProfile := ICCIGetIndProfile(ICInstanceGlobalsPtr(inst), index, thisID);
	end; (* ICGetIndProfile *)
	
	function ICGetProfileName(inst : ICInstance; thisID : ICProfileID; var name : Str255) : ICError;
	begin
	    ICGetProfileName := ICCIGetProfileName(ICInstanceGlobalsPtr(inst), thisID, name);
	end; (* ICGetProfileName *)
	
	function ICSetProfileName(inst : ICInstance; thisID : ICProfileID; name : Str255) : ICError;
	begin
	    ICSetProfileName := ICCISetProfileName(ICInstanceGlobalsPtr(inst), thisID, @name);
	end; (* ICSetProfileName *)
	
	function ICAddProfile(inst : ICInstance; prototypeID : ICProfileID; var newID : ICProfileID) : ICError;
	begin
	    ICAddProfile := ICCIAddProfile(ICInstanceGlobalsPtr(inst), prototypeID, newID);
	end; (* ICAddProfile *)
	
	function ICDeleteProfile(inst : ICInstance; thisID : ICProfileID) : ICError;
	begin
	    ICDeleteProfile := ICCIDeleteProfile(ICInstanceGlobalsPtr(inst), thisID);
	end; (* ICDeleteProfile *)
	
	(* ***** Interrupt Safe Routines ***** *)
	
	function ICRequiresInterruptSafe(inst : ICInstance) : ICError;
	begin
	    ICRequiresInterruptSafe := ICCIRequiresInterruptSafe(ICInstanceGlobalsPtr(inst));
	end; (* ICRequiresInterruptSafe *)
	
	function ICGetMappingInterruptSafe(inst : ICInstance; var mappingPref : Ptr; var mappingPrefSize : longint) : ICError;
	begin
	    ICGetMappingInterruptSafe := ICCIGetMappingInterruptSafe(ICInstanceGlobalsPtr(inst), mappingPref, mappingPrefSize);
	end; (* ICGetMappingInterruptSafe *)
	
	function ICGetSeedInterruptSafe(inst : ICInstance; var seed : longint) : ICError;
	begin
	    ICGetSeedInterruptSafe := ICCIGetSeedInterruptSafe(ICInstanceGlobalsPtr(inst), seed);
	end; (* ICGetSeedInterruptSafe *)
	

(* End ICLibrary.p *)

end. (* ICLibrary *)
