unit ICURLHandling;

(* Start Header *)

(* File:         ICURLHandling.p
 * Generated by: 1.0d8
 * For:          IC 2.1
 * On:           Monday, 4 October 1999, 0:53:14
 * 
 * This file is part of the Internet Configuration system and
 * is placed in the public domain for the benefit of all.
 *)

(* End Header *)

interface

	uses
		Types,
		Files, 
		QuickDraw, 
		Aliases, 
		Memory,
		Components,
		
		ICCore;

	(* ***** URL API Routines ***** *)
	
	(*	These interfaces are automagically generated by the Internet Config
		Interface Generator.  See "InternetConfig.p" for a description of
		the behaviour of these routines.
	*)

(* Start ICCIURL.p *)
	
	function ICCIEditPreferences(globals : ICInstanceGlobalsPtr; key : StringPtr) : ICError;

	function ICCIParseURL(globals : ICInstanceGlobalsPtr; hint : StringPtr; data : Ptr; len : longint; var selStart : longint; var selEnd : longint; url : Handle) : ICError;
	
	function ICCILaunchURL(globals : ICInstanceGlobalsPtr; hint : StringPtr; data : Ptr; len : longint; var selStart : longint; var selEnd : longint) : ICError;
	

(* End ICCIURL.p *)

implementation

	uses
		Resources,
		GestaltEqu, 
		OSUtils, 
		ToolUtils, 
		StandardFile, 
		AppleTalk, 
		Folders, 
		Processes, 
		Errors, 
		AppleEvents, 
		LowMem,
		TextUtils,

		ICDebug,
		ICStringLists,
		ICCommonSubs,
		ICResourceForkSanity, 
		ICExtensionResources;

	(* ***** Utilities to Support the AppleEvent Routines ***** *)

	function FindProcess (processCreator : OSType; var process: ProcessSerialNumber): Boolean;
		(* Looks for a running process based on its creator type.  If it finds it, it returns
			true and the processes PSN.
		*)
		var
			info: ProcessInfoRec;
	begin
		FindProcess := false;
		process.highLongOfPSN := 0;
		process.lowLongOfPSN := kNoProcess;
		info.processInfoLength := sizeof(ProcessInfoRec);
		info.processName := nil;
		info.processAppSpec := nil;
		while GetNextProcess(process) = noErr do begin
			if (GetProcessInformation(process, info) = noErr) &
							(info.processSignature = processCreator) then begin
				FindProcess := true;
				leave;
			end; (* if *)
		end; (* while *)
	end; (* FindProcess *)

	function ConfirmApplicationType(var fss: FSSpec; creator: OSType): OSErr;
		(* This routine confirms when the given FSSpec points to
			an application with the given type.  We use this to sanity
			check the information returned by the desktop database,
			ie avoid launching things that are in the DTDB but don't
			actually exist!
		*)
		var
			err: OSErr;
			info: FInfo;
	begin
		err := FSpGetFInfo(fss, info);
		if err = noErr then begin
			if (info.fdCreator <> creator) | not IsApplicationType(info.fdType) then begin
				err := afpItemNotFound;
			end; (* if *)
		end; (* if *)
		ConfirmApplicationType := err;
	end; (* ConfirmApplicationType *)
	
	function ConfirmModernICAp(var fss : FSSpec) : OSErr;
	
		function CompatibleVersionNumber(appVers : NumVersion; extensionVers : NumVersion) : Boolean;
			var
				result : Boolean;
		begin
			if appVers.majorRev > extensionVers.majorRev then begin
				result := true;		// 2.0 > 1.0
			end else if appVers.majorRev < extensionVers.majorRev then begin
				result := false;	// 1.0 < 2.0
			end else if appVers.minorAndBugRev > extensionVers.minorAndBugRev then begin
				result := true;		// 1.1 > 1.0, 1.0.1 > 1.0
			end else if appVers.minorAndBugRev < extensionVers.minorAndBugRev then begin
				result := false;	// 1.0 < 1.0, 1.0 < 1.0.1
			end else begin
				result := true;		// by fiat, within a bug fix release, all versions are equivalently compatible
			end; (* if *)
			CompatibleVersionNumber := result;
		end; (* CompatibleVersionNumber *)
		
		var
			err : OSErr;
			sysGlobals : ICSystemGlobalsPtr;
			refNum : integer;
			versH : VersRecHndl;
	begin
		err := noErr;
		sysGlobals := FindICSystemGlobals;
		if sysGlobals <> nil then begin

			// Only do our check if we're trying to launch the the ICAp
			// *and* IC is configured to use the IC app for ICEditPreferences.

			if sysGlobals^.icApplicationCreator = ICcreator then begin

				// Now for the main game.  Open up the app and check its version
				// number against the extensions version.

				refNum := FSpOpenResFile(fss, fsRdPerm);	// read-only means Im safe from getting an existing resource file refnum
				err := ResError;
				if err = noErr then begin
				
					versH := VersRecHndl(Get1Resource('vers', 1));
					if versH <> nil then begin
						if not CompatibleVersionNumber(versH^^.numericVersion, sysGlobals^.extensionVersion) then begin
							err := afpItemNotFound;
						end; (* if *)
					end; (* if *)
					
					CloseResFile(refNum);
					ICAssert(ResError = noErr);
				end; (* if *)
			end; (* if *)
		end; (* if *)
		ConfirmModernICAp := err;
	end; (* ConfirmModernICAp *)

	function ConfirmApplication(var fss : FSSpec; creator : OSType) : OSErr;
		(* Confirm the existence and validity of an application to launch.
			Start by confirming that there's really a file (of a reasonable
			type) there.  Then check to see whether the file is in the trash.
			No sneaky launching apps out of the trash in this version of IC!
			Also, if we're launching the IC application, make sure we reject
			any version of the application that's older than the installed
			component.
		*)
		var
			err : OSErr;
			inTrash : Boolean;
	begin
		err := ConfirmApplicationType(fss, creator);
		if err = noErr then begin
			err := IsFSSpecInTrash(fss, inTrash);
			if (err = noErr) & inTrash then begin
				err := afpItemNotFound;
			end; (* if *)
		end; (* if *)
		if (err = noErr) & (creator = ICcreator) then begin
			err := ConfirmModernICAp(fss);
		end; (* if *)
		ConfirmApplication := err;
	end; (* ConfirmApplication *)

	function ScanVolumeForApplication (vRefNum: integer; applCreator: OSType; var fss: FSSpec): OSErr;
		(* Scans the volume specified by vRefNum looking for the application
			with creator applCreator.  It sets up fss to point to the found application.
			
			Note that this routine doesn't just look for the first entry in the DTDB.  It
			tries to confirm whether that entry really points to an application and,
			if it doesn't, it keeps scanning through the DTDB for the volume.
		*)
		var
			err: OSErr;
			pbdt: DTPBRec;
			fileIndex: integer;
			found: Boolean;
	begin
		(* Get the path to the DTDB file. *)
		fss.name := '';
		pbdt.ioNamePtr := @fss.name;
		pbdt.ioVRefNum := vRefNum;
		err := PBDTGetPath(@pbdt);
		
		(* Now scan through the file looking for matching applications. *)
		if err = noErr then begin
			fileIndex := 1;
			found := false;
			repeat
				pbdt.ioIndex := fileIndex;
				pbdt.ioFileCreator := applCreator;
				err := PBDTGetAPPLSync(@pbdt);
				if err = noErr then begin
					(* We found a match, confirm that it exists.  Note that
						fss.name has already been setup by the PBDTGetAPPLSync
						call.
					*)
					fss.vRefNum := vRefNum;
					fss.parID := pbdt.ioAPPLParID;
					found := (ConfirmApplication(fss, applCreator) = noErr);
				end; (* if *)
				fileIndex := fileIndex + 1;
			until found or (err <> noErr);
		end; (* if *)
		ScanVolumeForApplication := err;
	end; (* ScanVolumeForApplication *)

	function FindApplication (applCreator: OSType; var fss: FSSpec): OSErr;
		(* This routine attempts to find an application someone in the file
			system with a matching creator.  It does this by scanning the
			desktop database on each available volume.
		*)

		function GetIndexedVolume (volumeIndex: integer; var vRefNum: integer): OSErr;
			(* This routine returns the vRefNum for the volumeIndex'th volume
				that's currently mounted.  It's in a separate procedure so that
				FindApplication doesn't waste stack space keeping a ParamBlockRec
				around while it calls ScanVolumeForApplication.
			*)
			var
				err: OSErr;
				pb: HParamBlockRec;
		begin
			pb.ioNamePtr := nil;
			pb.ioVRefNum := 0;
			pb.ioVolIndex := volumeIndex;
			err := PBHGetVInfoSync(@pb);
			if err = noErr then begin
				vRefNum := pb.ioVRefNum;
			end; (* if *)
			GetIndexedVolume := err;
		end; (* GetIndexedVolume *)

		var
			err: OSErr;
			volumeIndex: integer;
			vRefNum: integer;
			found: Boolean;
	begin
		(* Index through all the volumes calling ScanVolumeForApplication for a match. *)
		found := false;
		volumeIndex := 1;
		repeat
			vRefNum := 0;
			err := GetIndexedVolume(volumeIndex, vRefNum);
			if err = noErr then begin
				err := ScanVolumeForApplication(vRefNum, applCreator, fss);
				if err = noErr then begin
					found := true;
				end else begin
					err := noErr;		(* Swallow error so we continue with next volume. *)
				end; (* if *)
			end; (* if *)
			volumeIndex := volumeIndex + 1;
		until found or (err <> noErr);

		(* If we didn't find a match, always return the same error code. This
			makes it easier for our clients to display meaningful error messages.
		*)
		if not found then begin
			err := afpItemNotFound;
		end; (* if *)
		FindApplication := err;
	end; (* FindApplication *)

	function PrepareLaunchParamBlock (var theEvent: AppleEvent; var launchThis: LaunchParamBlockRec; var paramsDesc: AEDesc): ICError;
		(* Prepare a LaunchParamBlock for launching an application.  The application
			to launch has already been set up in launchThis.launchAppSpec.  This routine
			is responsible for filling out the other fields in the LaunchParamBlock, including
			the launchAppParameters, which is set up from theEvent parameter.
			The routine also returns the AEDesc form of theEvent so that the caller
			can dispose it.  It's all very ugly, but it works well in practice.
		*)
		var
			err : ICError;
	begin
		err := AECoerceDesc(theEvent, typeAppParameters, paramsDesc);
		if err = noErr then begin
			HLock(Handle(paramsDesc.dataHandle));
			launchThis.launchAppParameters := AppParametersPtr(paramsDesc.dataHandle^);
			launchThis.launchBlockID := extendedBlock;
			launchThis.launchEPBLength := extendedBlockLen;
			launchThis.launchFileFlags := 0;
			launchThis.launchControlFlags := launchContinue + launchNoFileFlags;
		end; (* if *)
		PrepareLaunchParamBlock := err;
	end; (* PrepareLaunchParamBlock *)

	function CreateGURLEvent (applCreator: OSType; urlH: Handle; var theEvent: AppleEvent): ICError;
		(* Create a GURL AppleEvent that targets the given application with urlH
			as the direct object.
		*)
		var
			err: ICError;
			err2: ICError;
			junk: ICError;
			targetAddress: AEDesc;
			s: signedByte;
	begin
		(* Create the event. *)
		err := AECreateDesc(typeApplSignature, @applCreator, sizeof(applCreator),
													targetAddress);
		err2 := AECreateAppleEvent('GURL', 'GURL', targetAddress, kAutoGenerateReturnID, 
													kAnyTransactionID, theEvent);
		if err = noErr then begin
			err := err2;
		end; (* if *)
		
		(* Add urlH as the direct object parameter. *)
		s := HGetState(urlH);
		HLock(urlH);
		err2 := AEPutParamPtr(theEvent, keyDirectObject, typeChar, urlH^, GetHandleSize(urlH));
		HSetState(urlH, s);
		if err = noErr then begin
			err := err2;
		end; (* if *)
		
		(* Clean up. *)
		if err <> noErr then begin
			junk := AEDisposeDesc(theEvent);
			ICAssert(junk = noErr);
		end; (* if *)
		junk := AEDisposeDesc(targetAddress);
		ICAssert(junk = noErr);

		CreateGURLEvent := err;
	end; (* CreateGURLEvent *)

	function CreateEditPrefEvent (creator : OSType; key: Str255; prefsFileSpec: FSSpec; var theEvent: AppleEvent): ICError;
		(* Create a Edit Preference AppleEvent that targets the given application with urlH
			as the direct object.
		*)
		var
			err: ICError;
			err2: ICError;
			junk: ICError;
			targetAddress: AEDesc;
	begin
		(* Create the event. *)
		err := AECreateDesc(typeApplSignature, @creator, sizeof(creator), targetAddress);
		err2 := AECreateAppleEvent(kICEditPreferenceEventClass, kICEditPreferenceEvent, 
								targetAddress, kAutoGenerateReturnID, kAnyTransactionID, theEvent);
		if err = noErr then begin
			err := err2;
		end; (* if *)
		
		(* Add key as the direct object parameter. *)
		err2 := AEPutParamPtr(theEvent, keyDirectObject, typeChar, @key[1], length(key));
		if err = noErr then begin
			err := err2;
		end; (* if *)
		
		(* Edit the destination file parameter. *)
		err2 := AEPutParamPtr(theEvent, keyICEditPreferenceDestination, typeFSS,
											@prefsFileSpec, sizeof(prefsFileSpec));
		if err = noErr then begin
			err := err2;
		end; (* if *)

		(* Clean up. *)
		if err <> noErr then begin
			junk := AEDisposeDesc(theEvent);
			ICAssert(junk = noErr);
		end; (* if *)
		junk := AEDisposeDesc(targetAddress);
		ICAssert(junk = noErr);

		CreateEditPrefEvent := err;
	end; (* CreateEditPrefEvent *)

	function LaunchFSSpec (var applFileSpec: FSSpec; theEvent: AppleEvent): ICError;
		(* Launch application denoted by applFileSpec passing it the AppleEvent
			as its application parameter.
		*)
		var
			err: ICError;
			junk: ICError;
			launchThis: LaunchParamBlockRec;
			paramsDesc: AEDesc;
	begin
		(* Setup the launch parameter block. *)
		launchThis.launchAppSpec := @applFileSpec;
		err := PrepareLaunchParamBlock(theEvent, launchThis, paramsDesc);
		
		(* Launch the application. *)
		if err = noErr then begin
			err := LaunchApplication(@launchThis);
		end; (* if *)
		
		(* If it fails because we're out of memory, try launching the application in
			the minimum partition.
		*)
		if err = memFullErr then begin
			launchThis.launchControlFlags := bor(launchThis.launchControlFlags, launchUseMinimum);
			err := LaunchApplication(@launchThis);
		end; (* if *)
		
		(* Dispose the application parameters AEDesc that was created by
			PrepareLaunchParamBlock.
		*)
		junk := AEDisposeDesc(paramsDesc);
		ICAssert(junk = noErr);

		LaunchFSSpec := err;
	end; (* LaunchFSSpec *)

	function SendEvent (theEvent: AppleEvent; applCreator: OSType): ICError;
		(* Send an AppleEvent to a given application, denoted by its
			creator code.  The implementation handles two cases: if the
			application is already running, it sends it the AppleEvent directly.
			If the application is not running, it launches it with the AppleEvent
			packaged into the application parameters.
		*)
		var
			err: ICError;
			junk: ICError;
			applPSN: ProcessSerialNumber;
			applFileSpec: FSSpec;
			reply: AppleEvent;
	begin
		if FindProcess(applCreator, applPSN) then begin
			junk := SetFrontProcess(applPSN);
			err := AESend(theEvent, reply, kAENoReply, kAEHighPriority, kNoTimeOut, nil, nil);
		end else begin
			err := FindApplication(applCreator, applFileSpec);
			if err = noErr then begin
				err := LaunchFSSpec(applFileSpec, theEvent);
			end; (* if *)
		end; (* if *)
		SendEvent := err;
	end; (* SendEvent *)

	(* ***** AppleEvent Routines ***** *)

	function ICUEditPreferences (creator : OSType; key: Str255; prefsFileSpec: FSSpec): ICError;
		(* Implements the core of the ICEditPreferences call.  Sends
			an Edit Preferences AppleEvent to the Internet Config application,
			telling it to edit the given key and preferences file.
		*)
		var
			err: ICError;
			junk: ICError;
			theEvent: AppleEvent;
	begin
		err := CreateEditPrefEvent(creator, key, prefsFileSpec, theEvent);
		if err = noErr then begin
			err := SendEvent(theEvent, creator);
		end; (* if *)
		junk := AEDisposeDesc(theEvent);
		ICAssert(junk = noErr);
		ICUEditPreferences := err;
	end; (* ICUEditPreferences *)

	function ICULaunchURL (helperCreator: OSType; urlH: Handle): ICError;
		(* Implements the core of the ICLaunchURL call.  Sends
			an GURL AppleEvent to the specified helper application,
			telling it to launch the given URL.
		*)
		var
			err: ICError;
			junk: ICError;
			theEvent: AppleEvent;
	begin
		err := CreateGURLEvent(helperCreator, urlH, theEvent);
		if err = noErr then begin
			err := SendEvent(theEvent, helperCreator);
		end; (* if *)
		junk := AEDisposeDesc(theEvent);
		ICAssert(junk = noErr);
		ICULaunchURL := err;
	end; (* ICULaunchURL *)

	(* ***** User Interface Stuff ***** *)

	function ICCIEditPreferences (globals : ICInstanceGlobalsPtr; key: StringPtr): ICError;
		(* See comment in ICAPI.p. *)
		var
			err: ICError;
			copiedKey: Str255;
	begin
		(* Check parameters. *)
		copiedKey := '';
		err := noErr;
		if (err = noErr) and (key <> nil) then begin
			BlockMoveData(key, @copiedKey, ord(key^[0]) + 1);
		end; (* if *)

		(* Pass the info along to ICUEditPreferences which does all the work. *)
		if err = noErr then begin
			err := ICUEditPreferences(globals^.sysGlobals^.icApplicationCreator, copiedKey, globals^.instanceFileSpec^);
		end; (* if *)
		ICCIEditPreferences := err;
	end; (* ICCIEditPreferences *)

	(* ***** URL Handling Utilities ***** *)

	(* URL Parsing Algorithm

		The following is a short summary of the URL parsing algorithm:
		
		1.		If there is a selection skip to Step 4.
		2.		Expand selection in both directions to end of word (never skip
				a bracket though).
		3.		If either end has a bracket then expand other end to search for 
				a matching bracket.
		4.		Strip trailing and leading whitespace.
		5. 		Strip whitespace CR whitespace.
		6. 		Take off the outside brackets or ...
		7. 		... strip trailing "." unless there was originally a selection.
		8. 		Remove leading "URL:"
		9. 		Extract scheme by looking forwards for ":"
		10. 	If the URL starts with a "www.", override the hint supplied by
				the caller with "http://".
		11. 	If no scheme then either prepend "hint:" or just "hint" depending
				on whether hint already has a ":".
	*)

	(* This array is used to access the characters in the URL. *)
	const
		kVeryBig = 100000;
	type
		DataArray = packed array[0..kVeryBig] of char;
		DataPtr = ^DataArray;
		DataHandle = ^DataPtr;

	(* These sets represent the various types of characters in a URL. *)

	const
		kForwardExpandTerminateSet = [' ', '<', '[', '(', '{', '"', '''', '', '', chr(9), chr(13)];
		(* These characters will terminate an expansion (ie Step 2 in the algorithm
			above) in the forward direction (ie towards the front of the URL).
		*)

		kForwardBracketSet = ['<', '[', '(', '{', '"', '''', '', ''];
		(* This set contains all the characters that are considered forward
			brackets (see Step 2 and 3 in the algorithm above).
		*)

		kBackwardExpandTerminateSet = [' ', '>', ']', ')', ')', '"', '''', '', '', chr(9), chr(13)];
		(* These characters will terminate an expansion (ie Step 2 in the algorithm
			above) in the backward direction (ie towards the end of the URL).
		*)
		
		kBackwardBracketSet = ['>', ']', ')', '}', '"', '''', '', ''];
		(* This set contains all the characters that are considered backward
			brackets (see Step 2 and 3 in the algorithm above).
		*)
		
		kWhitespaceSet = [' ', chr(9)];
		kWhitespaceReturnSet = [' ', chr(9), chr(13)];
		(* These characters are considered whitespace by Steps 4 and
			5 of the algorithm above.
		*)

	function MatchingBracket (ch: char): char;
		(* Returns the matching bracket for the input character,
			eg MatchingBracket('{') = '}' and MatchingBracket('}') = '{'.
			Despite the size of the source code, this generates
			a surprisingly small executable.
		*)
	begin
		case ch of
			'<': 
				MatchingBracket := '>';
			'>': 
				MatchingBracket := '<';
			'[': 
				MatchingBracket := ']';
			']': 
				MatchingBracket := '[';
			'(': 
				MatchingBracket := ')';
			')': 
				MatchingBracket := '(';
			'{': 
				MatchingBracket := '}';
			'}': 
				MatchingBracket := '{';
			'': 
				MatchingBracket := '';
			'': 
				MatchingBracket := '';
			'': 
				MatchingBracket := '';
			'': 
				MatchingBracket := '';
			otherwise
				MatchingBracket := ch;
		end; (* case *)
	end; (* MatchingBracket *)

	function ExpandSelection (dataP: DataPtr; len: longint; var selStart, selEnd: longint): ICError;
		(* Expands the selection denoted by selStart and selEnd in the URL
			text denoted by dataP and len.  This implemenst Step 2 and 3 of the
			algorithm given above.
		*)
		var
			err: ICError;
			found: Boolean;
			searchChar: char;
	begin
		err := noErr;

		(* Expand leading selection looking for word break. If we hit a bracket,
			stop, but also include the bracket in the selection.
		*)
		while (selStart > 0) & not (dataP^[selStart - 1] in kForwardExpandTerminateSet) do begin
			selStart := selStart - 1;
		end; (* while *)
		if (selStart > 0) & (dataP^[selStart - 1] in kForwardBracketSet) then begin
			selStart := selStart - 1;
		end; (* if *)
		
		(* Expand the trailing selection looking for work break.  If we hit a bracket,
			stop, but also include the bracket in the selection.
		*)
		while (selEnd < len) & not (dataP^[selEnd] in kBackwardExpandTerminateSet) do begin
			selEnd := selEnd + 1;
		end; (* while *)
		if (selEnd < len) & (dataP^[selEnd] in kBackwardBracketSet) then begin
			selEnd := selEnd + 1;
		end; (* if *)

		(* If first character was a bracket, expand the trailing selection
			to meet the matching bracket.
		*)
		if dataP^[selStart] in kForwardBracketSet then begin
			searchChar := MatchingBracket(dataP^[selStart]);
			found := false;
			while not found and (selEnd - 1 < len) do begin
				found := (dataP^[selEnd - 1] = searchChar);
				if not found then begin
					selEnd := selEnd + 1;
				end; (* if *)
			end; (* while *)
			if not found then begin
				err := icNoURLErr;
			end; (* if *)
		end; (* if *)

		(* If last character was a bracket, expand leading selection to meet
			matching bracket.
		*)
		if (err = noErr) & (selEnd > 0) & (dataP^[selEnd - 1] in kBackwardBracketSet) then begin
			searchChar := MatchingBracket(dataP^[selEnd - 1]);
			found := (dataP^[selStart] = searchChar);
			while not found and (selStart >= 0) do begin
				found := (dataP^[selStart] = searchChar);
				if not found then begin
					selStart := selStart - 1;
				end; (* if *)
			end; (* if *)
			if not found then begin
				err := icNoURLErr;
			end; (* if *)
		end; (* if *)

		ExpandSelection := err;
	end; (* ExpandSelection *)

	function ShrinkSelection (dataP: DataPtr; len: longint; var selStart, selEnd: longint): ICError;
		(* Shrink the leading and trailing edges of the selection denoted by
			selStart and selEnd in the URL text denoted by dataP and len.
			This implemenst Step 4 of the algorithm given above.
		*)
	begin
		(*Strip leading whitespace. *)
		while (selStart < len) & (dataP^[selStart] in kWhitespaceSet) do begin
			selStart := selStart + 1;
		end; (* while *)
		(* Strip trailing whitespace. *)
		while (selEnd > 0) & (dataP^[selEnd - 1] in kWhitespaceSet) do begin
			selEnd := selEnd - 1;
		end; (* while *)
		ShrinkSelection := noErr;
	end; (* ShrinkSelection *)

	function StripReturns (urlH: DataHandle): ICError;
		(* Removes any sequence of <whitespace> <cr> <whitespace> from urlH.
			This is Step 5 of the algorithm described above.
		*)
		var
			err: ICError;
			sourceSize: longint;
			sourceIndex: longint;
			destIndex: longint;
	begin
		sourceSize := GetHandleSize(Handle(urlH));
		sourceIndex := 0;
		destIndex := 0;

		(* Skip down the handle, copying the character at sourceIndex to
			destIndex except when meeting CR where we do special things.
		*)
		while sourceIndex < sourceSize do begin

			if urlH^^[sourceIndex] = chr(13) then begin
				(* We've hit a CR.  Move destIndex back to point to previous
					non-whitespace character.  Then move sourceIndex forwards
					to next non-whitespace, skipping both normal whitespace
					and CRs.
				*)
				while (destIndex > 0) & (urlH^^[destIndex - 1] in kWhitespaceSet) do begin
					destIndex := destIndex - 1;
				end; (* while *)
				while (sourceIndex < sourceSize) & (urlH^^[sourceIndex] in kWhitespaceReturnSet) do begin
					sourceIndex := sourceIndex + 1;
				end; (* while *)
			end; (* case *)

			(* The previous code could have moved sourceIndex beyond sourceSize,
				so check again before copying a bit from sourceIndex to destIndex.
			*)
			if sourceIndex < sourceSize then begin
				urlH^^[destIndex] := urlH^^[sourceIndex];
				sourceIndex := sourceIndex + 1;
				destIndex := destIndex + 1;
			end; (* if *)
		end; (* while *)

		(* Resize the handle to the number of bytes that we copied. *)
		SetHandleSize(Handle(urlH), destIndex);
		err := MemError;
		if (err = noErr) & (GetHandleSize(Handle(urlH)) = 0) then begin
			err := icNoURLErr;
		end; (* if *)

		StripReturns := err;
	end; (* StripReturns *)

	function FindScheme (urlH: Handle; scheme: StringPtr): ICError;
		(* This routine extracts the scheme part from the URL
			by search through the handle for the first ":" and then
			copying all of the characters from the start of the
			URL to just before the ":" out into the scheme string.
			If it can't find a ":", it returns an error.
			If scheme is nil, it does not copy the string out,
			so only the error return is significant.
			FindScheme implements Step 9 of the URL parsing algorithm.
		*)
		var
			err: ICError;
			tmp: Str15;
			ndx: longint;
	begin
		err := noErr;
		tmp := ':';
		ndx := Munger(Handle(urlH), 0, @tmp[1], length(tmp), nil, 0);
		if (ndx < 0) or (ndx > 255) then begin
			err := icNoURLErr;
		end; (* if *)
		if (err = noErr) and (scheme <> nil) then begin
			scheme^[0] := chr(ndx);
			BlockMoveData(urlH^, @scheme^[1], ndx);
		end; (* if *)
		FindScheme := err;
	end; (* FindScheme *)

	(* ***** URL Handling ***** *)

	function ICCIParseURL (globals : ICInstanceGlobalsPtr; hint: StringPtr; data: Ptr; len: longint; var selStart: longint; var selEnd: longint; url: Handle): ICError;
		(* See comment in ICAPI.p.  Note the sneaky use of aliases in this
			routine.  urlH and url are the same handle, its just that urlH
			is of type DataHandle whereas url is of type Handle.  The code
			uses them interchangably to avoid a ridiculous number of type
			casts, but it can get confusing.
		*)
		var
			err: ICError;
			dataP: DataPtr;
			urlH: DataHandle;
			explicitSelection: Boolean;
			copiedHint: Str255;
			tmp: Str15;
			junklong: longint;
	begin
		{$unused globals}
		(* Setup some locals which are retyped versions of the input parameters. *)
		dataP := DataPtr(data);
		urlH := DataHandle(url);
		explicitSelection := true;
		
		(* Check the parameters. *)
		err := noErr;
		ICParamCheck(err, data <> nil, 'ICParseURL: data must not be nil');
		ICParamCheck(err, url <> nil, 'ICParseURL: url must not be nil');
		ICParamCheck(err, url^ <> nil, 'ICParseURL: url has been purged');
		ICParamCheck(err, len > 0, 'ICParseURL: len must be positive');
		ICParamCheck(err, selStart >= 0, 'ICParseURL: selStart must not be negative');
		ICParamCheck(err, selEnd >= 0, 'ICParseURL: selEnd must not be negative');
		ICParamCheck(err, selStart <= len, 'ICParseURL: selStart must <= len');
		ICParamCheck(err, selEnd <= len, 'ICParseURL: selEnd must <= len');
		ICParamCheck(err, selStart <= selEnd, 'ICParseURL: selEnd must be before selEnd');
		
		(* If theres not explicit selection, expand the insertion point.  This is
			Step 2 and 3 of the algorithm.
		*)
		if (err = noErr) and (selStart = selEnd) then begin
			explicitSelection := false;
			err := ExpandSelection(dataP, len, selStart, selEnd);
		end; (* if *)

		(* Remove any leading or trailing whitespace -- Step 4 of the algorithm. *)
		if err = noErr then begin
			err := ShrinkSelection(dataP, len, selStart, selEnd);
		end; (* if *)
		if (err = noErr) and (selStart >= selEnd) then begin
			(* No selection remains.  We dont have a URL. *)
			err := icNoURLErr;
		end; (* if *)

		(* Copy the selection (ie the URL) into url handle. *)
		if err = noErr then begin
			err := PtrToXHand(@dataP^[selStart], url, selEnd - selStart);
		end; (* if *)

		(* Step 5 of the algorithm is to remove any <whitespace> <cr> <whitespace>
			sequences.
		*)
		if err = noErr then begin
			err := StripReturns(urlH);
		end; (* if *)

		if err = noErr then begin
			if (urlH^^[0] in kForwardBracketSet) &
						(urlH^^[GetHandleSize(Handle(urlH)) - 1] = MatchingBracket(urlH^^[0])) then begin
				(* If the first character of the URL is a bracket and the last
					character is the matching opposite bracket, remove the brackets.
					Step 6 of the algorithm.
				*)
				SetHandleSize(url, GetHandleSize(url) - 1);
				junklong := Munger(url, 0, nil, 1, Ptr(-1), 0);	(* trim off first character *)
				ICAssert(MemError = noErr);		(* shrinking handle, so only assert error code *)
			end else if not explicitSelection then begin
				(* If the last character is a "." and there was no explicit selection,
					remove it.  Step 7 of the algorithm.  This is designed to handle
					the case where losers put URLs at the end of sentences and
					fail to enclose them in brackets, for example
					http://redback.cs.uwa.edu.au/Quinn/.
				*)
				if urlH^^[GetHandleSize(Handle(urlH)) - 1] = '.' then begin
					SetHandleSize(url, GetHandleSize(url) - 1);
				end; (* if *)
			end; (* if *)
			
			(* Step 8 is to trim off a leading "URL:" if there is one.  Note the use
				of HUnlock vs HSetState.  This is cool because we know that
				the handle should be unlocked becauses we created it.
			*)
			tmp := 'URL:';
			HLock(url);
			if (GetHandleSize(url) >= length(tmp)) &
						(IdenticalText(url^, @tmp[1], length(tmp), length(tmp), nil) = 0) then begin
				(* Unlock urlH cause Munger is going to want it that way. *)
				HUnlock(url);
				(* Trim the 'URL:' characters off the front of the URL. *)
				junklong := Munger(url, 0, nil, length(tmp), Ptr(-1), 0);
				ICAssert(MemError = noErr);		(* shrinking handle, so only assert error code *)
			end; (* if *)
			HUnlock(url);

			(* Search for the URL scheme -- Step 9 of the algorithm. *)
			if FindScheme(url, nil) <> noErr then begin
				// Failed to find the scheme (ie a ":" in first 256 bytes).
				// Check for a leading "www."; if we find that, were
				// going to assume its an HTTP URL and override the provided
				// hint with "http://".  Step 10 of the algorithm.
				
				tmp := 'www.';
				if (GetHandleSize(url) >= length(tmp)) & 
							(IdenticalText(url^, @tmp[1], length(tmp), length(tmp), nil) = 0) then begin

					// The URL starts with a "www.".  Were going to assume
					// its a web URL!
					
					copiedHint := "http://";
				end else begin
				
					// The URL did not start with a "www.", so were going
					// to use the hint provided by the application, if there
					// is one.
					
					if (hint = nil) | (length(hint^) = 0) then begin
						copiedHint := '';
					end else begin

						// Copy the hint into a local variable (see the note about
						// copiedKey at the top of this file).
						
						BlockMoveData(hint, @copiedHint, ord(length(hint^)) + 1);
						if pos(':', copiedHint) = 0 then begin
							// Only add a colon if the hint doesnt already have one.
							copiedHint := concat(copiedHint, ':');
						end; (* if *)
					end; (* if *)
				end; (* if *)
				
				// Now use the hint to complete the URL.
				
				if copiedHint = '' then begin
					// Whoops, theres no hint to prepend, we so bail out.
					err := icNoURLErr;
				end else begin
				
					// Prepend the hint at the front the URL.  This is
					// algorithm Step 11.
					
					junklong := Munger(url, 0, nil, 0, @copiedHint[1], length(copiedHint));
					err := MemError;
				end; (* if *)
			end; (* if *)
		end; (* if *)
		ICCIParseURL := err;
	end; (* ICCIParseURL *)

	function ICCILaunchURL (globals : ICInstanceGlobalsPtr; hint: StringPtr; data: Ptr; len: longint; var selStart: longint; var selEnd: longint): ICError;
		(* See comment in ICAPI.p. *)
		var
			err: ICError;
			urlH: Handle;
			helper: ICAppSpec;
			scheme: Str255;
			junkAttr: longint;
			size: longint;
	begin
		(* Parse the URL out of the text into a handle. *)
		urlH := NewHandle(0);
		err := MemError;
		if err = noErr then begin
		
			// hint^ trips a nil pointer check if "hint" is nil.  In general, I guard against
			// being passed nil strings because C programmers do it all the time.  However,
			// its really hard to prevent this automatic check without copying the string
			// to a local variable, and that would burn stack space just to get around a
			// debug check.  So I just disable the nil check generation for this single line.
			//
			// Quinn, 27 Apr 1999, IC 2.1d1
			
			{$pragmac trapnil off}
			err := ICParseURL(globals^.target, hint^, data, len, selStart, selEnd, urlH);
			{$pragmac trapnil reset}
		end; (* if *)
		
		(* Lookup the scheme associated with the URL, then find the helper
			preference for that scheme and send that application
			the GURL AppleEvent.
		*)
		if err = noErr then begin
			err := FindScheme(urlH, @scheme);
		end; (* if *)
		if err = noErr then begin
			size := sizeof(helper);
			err := ICGetPref(globals^.target, concat(kICHelper, scheme), junkAttr, @helper, size);
			if (err = icPrefNotFoundErr) &
						(EqualString(scheme, 'https', false, true) |
								EqualString(scheme, 'file', false, true)) then begin
				(* If the user has no preference for "https" or "file" URLs, automagically
					pass them to the "http" helper.
				*)
				size := sizeof(helper);
				err := ICGetPref(globals^.target, concat(kICHelper, 'http'), junkAttr, @helper, size);
			end; (* if *)
		end; (* if *)
		if err = noErr then begin
			err := ICULaunchURL(helper.fCreator, urlH);
		end; (* if *)
		
		(* Clean up. *)
		if urlH <> nil then begin
			DisposeHandle(urlH);
		end; (* if *)
		
		{$ifc icDebug}
			if err = noPortErr then begin
				ICDebugStr('ICLaunchURL: About to return error -903; set the "is high-level event aware" bit in your SIZE resource');
			end; (* if *)
		{$endc}
		
		ICCILaunchURL := err;
	end; (* ICCILaunchURL *)

end. (* ICURLHandling *)
