/*	This file is part of the Internet Configuration system and is placed in the public domain for the benefit of all.

	Grumble, grumble, grumble.  This file contains an implementation of
	certain Carbon APIs as source which must be compiled into into the
	application when compiling for pre-Carbon.  It has to be written in
	C because some *utter gumby* (I know who you are) decided that
	certain new APIs should be exported as a C function rather than
	a Pascal function.  This causes all sorts of problems if you
	try to write the compatibility layer in Pascal because, while
	Pascal can call C functions, it can't really export them very well.
*/

#include <Scrap.h>
#include <MacMemory.h>

#include "ICDebug.h"
#include "ICCommonSubs.h"

// ***** Scrap *****

// Carbon supports the ScrapRef data type as a way of describing
// multiple scraps.  Carbon 1.0 doesn't actually support multiple
// scraps, so our emulation layer doesn't have to either.  So
// we always return the ScrapRef of kMagicScrapValue and all
// the entry points check for that value, just to keep the client
// honest.

enum {
	kMagicScrapValue = 666
};

extern OSStatus ClearCurrentScrap(void)
{
	OSStatus err;

	err = ZeroScrap();
	ICAssert(err <= noErr);
	return err;
}

extern OSStatus GetCurrentScrap(ScrapRef *scrap)
{
	*scrap = (ScrapRef) kMagicScrapValue;
	return noErr;
}

extern OSStatus PutScrapFlavor(ScrapRef scrap, ScrapFlavorType flavorType, ScrapFlavorFlags flavorFlags, Size flavorSize, const void *flavorData)
{
	OSStatus err;

	#pragma unused(scrap)
	ICAssert( (long) scrap == kMagicScrapValue);
	#pragma unused(flavorFlags)
	ICAssert(flavorFlags == kScrapFlavorMaskNone);
	err = PutScrap(flavorSize, flavorType, flavorData);
	ICAssert(err <= noErr);
	return err;
}

extern OSStatus GetScrapFlavorFlags(ScrapRef scrap, ScrapFlavorType flavorType, ScrapFlavorFlags *flavorFlags)
{
	SInt32 result;
	SInt32 offset;

	#pragma unused(scrap)
	ICAssert( (long) scrap == kMagicScrapValue);
	*flavorFlags = kScrapFlavorMaskNone;
	offset = 0;
	result = GetScrap(nil, flavorType, &offset);
	if (result > 0) {
		result = noErr;
	}
	return result;
}

extern OSStatus GetScrapFlavorSize(ScrapRef scrap, ScrapFlavorType flavorType, Size *byteCount)
{
	SInt32 result;
	SInt32 offset;

	#pragma unused(scrap)
	ICAssert( (long ) scrap == kMagicScrapValue);
	*byteCount = 0;
	offset = 0;
	result = GetScrap(nil, flavorType, &offset);
	if (result > 0) {
		*byteCount = result;
		result = noErr;
	}
	return result;
}
	
extern OSStatus GetScrapFlavorData(ScrapRef scrap, ScrapFlavorType flavorType, Size *byteCount, void *destination)
{
	OSStatus err;
	Handle scrapH;
	SInt32 offset;
	Size bytesToCopy;

	#pragma unused(scrap)
	ICAssert( (long) scrap == kMagicScrapValue);
	bytesToCopy = *byteCount;
	*byteCount = 0;
	scrapH = NewHandle(0);
	err = CheckMemError(scrapH);
	if (err == noErr) {
		offset = 0;
		err = GetScrap(scrapH, flavorType, &offset);
		if (err > 0) {
			*byteCount = err;
			err = noErr;
		}
		if (err == noErr) {
			if (*byteCount < bytesToCopy) {
				bytesToCopy = *byteCount;
			}
			BlockMoveData(*scrapH, destination, bytesToCopy);
		}
	}
		
	// Clean up.
	
	if (scrapH != nil) {
		DisposeHandle(scrapH);
		ICAssert(MemError() == noErr);
	}
	
	return err;
}
